/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/* Implementation of the javax.activity.coordination.ServiceManager   */
/* interface. A ServiceManager is a factory for SignalSets and        */
/* other objects used by a high-level service.                        */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.hls;

import javax.activity.*;
import javax.activity.coordination.*;
import javax.activity.propertygroup.*;


import java.util.Hashtable;
import java.util.HashMap;
import java.util.Vector;
import java.util.Properties;
import java.lang.reflect.*;



/**
 * An implementation of the <code>ServiceManager</code> interface providing
 * the link to <code>ActivityManager</code> function.
 * Each ServiceManager gets its own ActivityManager instance and so all
 * calls to the ActivityManager must pass through this class.
 * The ActivityManager class that is used is passed via the property
 * javax.activity.ActivityManager.  It must have a public default constructor.
 */

public final class ServiceManagerImpl implements ServiceManager
   {
   /**
    * Constructor which does no registration
    * @param serviceName   The name under which the service is registered
    * @param testLog       an object used for logging test status
    *
    * @exception various  All exceptions are propagated to the caller
    *     to facilitate tracing error conditions
    */
   public ServiceManagerImpl (String serviceName, TestLog testLog)
      throws ClassNotFoundException, InstantiationException,
      IllegalAccessException
      {
      Trace.out ("ServiceManagerImpl constructor " + serviceName);
      _testLog = testLog;
      _si = new ServiceInformation (
         serviceName, "", (java.io.Serializable) null);

      // locate the ActivityManager interface
      _userActivity = getActivityManagerInstance ();
      }  // end c'tor


   /**
    * Allow ServiceManager to change its contextGroup and also
    * register/re-register with the UserActivity/ActivityManager
    * @param serviceName   The name under which the service is registered
    * @param contextGroup  The name of the context group that this service
    *     produces Activities under.
    * @param testLog       an object used for logging test status
    * @param register      a flag to indicate wheter to register with
    *     the userActivity interface
    *
    * @exception various   All exceptions are propagated to the caller
    *     to facilitate tracing error conditions
    */
   public void setContextGroup (String serviceName, String contextGroup,
      boolean register)
      throws ClassNotFoundException, InstantiationException,
      IllegalAccessException, ServiceAlreadyRegisteredException,
      PropertyGroupUnknownException, SystemException
      {
      Trace.out ("ServiceManagerImpl.setContextGroup: " + serviceName +
         " contextGroup: " + contextGroup + " register: " + register);
      _si = new ServiceInformation (
         serviceName, contextGroup, (java.io.Serializable) null);

      if (register)
         {
         if (_userActivity == null)
            {
            _userActivity = getActivityManagerInstance ();
            }  // end if

         Trace.out (">>> UserActivity.registerService");
         _userActivity.registerService (this);
         Trace.out ("<<< UserActivity.registerService");
         }  // end if

      }  // end setContextGroup

   /**
    * Constructor which locates the ActivityManager class and registers
    * itself using the ServiceInformation passed in.
    * @param serviceName   The name under which the service is registered
    * @param contextGroup  The name of the context group that this service
    *     produces Activities under.
    * @param testLog  an object used for logging test status
    *
    * @exception various  All exceptions are propagated to the caller
    *     to facilitate tracing error conditions
    */
   public ServiceManagerImpl (String serviceName, String contextGroup,
      TestLog testLog)
      throws ClassNotFoundException, InstantiationException,
      IllegalAccessException, ServiceAlreadyRegisteredException,
      PropertyGroupUnknownException, SystemException
      {
      Trace.out ("ServiceManagerImpl constructor " + serviceName
         + " contextGroup: " + contextGroup);

      _testLog = testLog;
      _si = new ServiceInformation (
         serviceName, contextGroup, (java.io.Serializable) null);

      _userActivity = getActivityManagerInstance ();

         Trace.out (">>> UserActivity.registerService");
      _userActivity.registerService (this);
         Trace.out ("<<< UserActivity.registerService");
      }  // end c'tor

   /**
    * Locate and return an ActivityManager object using classes
    * specified in property files and introspection.
    *
    */
   private ActivityManager getActivityManagerInstance ()
      throws ClassNotFoundException, InstantiationException,
      IllegalAccessException
      {
      Trace.out ("ServiceManagerImpl.getActivityManagerInstance");
      ActivityManager result = null;

      // locate the ActivityManager interface using ActivityManager property
      Properties props   = System.getProperties ();
      String actmgrClass = System.getProperty (
         "javax.activity.ActivityManager");
      if (actmgrClass != null)
         {
         Class c = Class.forName (actmgrClass);
         result = (ActivityManager) c.newInstance ();
         }
      else  // try ActivityManagerFactory property
         {
         actmgrClass = System.getProperty (
            "javax.activity.ActivityManagerFactory");
         if (actmgrClass != null)
            {
            Trace.out ( " ... found ActivityManagerFactory property");
            Class  c = Class.forName (actmgrClass);
            Trace.out (" ... created ActivityManagerFactory class");
            Method [] methods = c.getMethods ();
            for (int i = 0; i < methods.length; i++)
               {
               Method m = methods [i];
               if (m.getName ().equals ("getObjectInstance"))
                  {
                  Trace.out (" ... found getObjectInstance method");
                  // now get parameters and create object array
                  Class [] ptypes = m.getParameterTypes ();
                  Trace.out (" ... no. parameters = " + ptypes.length);
                  Object [] parms = new Object [ptypes.length];
                  for (int j = 0; j < parms.length; j++)
                     parms [j] = null;

                  // create factory instance
                  Object o = c.newInstance ();
                  // invoke
                  try {
                     result = (ActivityManager) m.invoke (o, parms);
                     }
                  catch (InvocationTargetException e)
                     {
                     Trace.out ("Error creating ActivityManager: ");
                     Trace.exception (e);
                     throw new InstantiationException (
                        "Error creating ActivityManager: " + e);
                     }
                  break;
                  }  // end if

               }  // end for

            }  // end if

         } // end else

      if (result == null)
         throw new InstantiationException (
            "Unable to find factory/getObjectInstance method");

      return result;
      }  // end ServiceManagerImpl.getActivityManagerInstance



   /**
    * Add a <code>SignalSet</code> to the list of SignalSets that this
    * <code>ServiceManager</code> supports.
    * @param sigset      the <code>SignalSet</code> to add.
    * @param completion  if true, then this SignalSet is the <i>completion</i>
    *    SignalSet
    *
    */
   public void addSignalSet (SignalSet sigset, boolean completion)
      {
      String  ssName = null;
      if (sigset != null)
         ssName = sigset.getSignalSetName ();
      Trace.out ("ServiceManagerImpl.addSignalSet " + ssName);
      _signalSets.put (ssName, sigset);
      if (completion)
         _cssName = ssName;
      }  // end addSignalSet


   /**
    * Add a <code>PropertyGroupManager</code> to the list of PGMs that this
    * <code>ServiceManager</code> supports.
    * @param name     the name of the PropertyGroupManager being added
    * @param pgm      the <code>PropertyGroupManager</code> to add.
    *
    */
   public void addPropertyGroupManager (String name, PropertyGroupManager pgm)
      {
      Trace.out ("ServiceManagerImpl.addPropertyGroupManager " + name);
      _pgms.put (name, pgm);
      _pgNames = (String[])   _pgms.keySet ().toArray (_pgNames);
      }  // end addPropertyGroupManager


// *************************************
// *** ACTIVITYMANAGER  METHODS ********
// *************************************


   public void begin (int timeout) throws InvalidStateException,
      ServiceNotRegisteredException, TimeoutRangeException, SystemException
      {
      Trace.out ("ServiceManagerImpl.begin " + timeout);
      Trace.out (">>> UserActivity.begin");
      _userActivity.begin (timeout);
      Trace.out ("<<< UserActivity.begin");
      }  // end begin


   public ActivityCoordinator recreate(GlobalId activity, GlobalId parent, boolean resume, RecreateData recData)
      throws ServiceNotRegisteredException, ActivityCompletedException,
      IllegalArgumentException, SystemException
      {
      Trace.out ("ServiceManagerImpl.recreate ");
      Trace.out (">>> UserActivity.recreate");
      ActivityCoordinator result = _userActivity.recreate (activity, parent, resume, recData);
      Trace.out ("<<< UserActivity.recreate");
      return result;
      }

   public Outcome complete() throws NoActivityException,
      ActivityPendingException, ContextPendingException,
      ActivityNotProcessedException, ServiceNotRegisteredException,
      NotOriginatorException, SystemException
      {
      Trace.out ("ServiceManagerImpl.complete");
      Trace.out (">>> UserActivity.complete");
      Outcome outcome = _userActivity.complete ();
      Trace.out ("<<< UserActivity.complete");
      return outcome;
      }  // end complete

   public Outcome completeWithStatus (int completionStatus)
      throws NoActivityException, ActivityPendingException,
      InvalidStateException, ContextPendingException,
      ActivityNotProcessedException, ServiceNotRegisteredException,
      NotOriginatorException, SystemException
      {
      Trace.out ("ServiceManagerImpl.completeWithStatus " + completionStatus);
      Trace.out (">>> UserActivity.completeWithStatus");
      Outcome outcome = _userActivity.completeWithStatus (completionStatus);
      Trace.out ("<<< UserActivity.completeWithStatus");
      return outcome;
      }  // end completeWithStatus

   public Outcome broadcast (String signalSetName)
      throws NoActivityException, SignalSetUnknownException,
      ActivityNotProcessedException, ServiceNotRegisteredException,
      InvalidActivityException, SystemException
      {
      Trace.out ("ServiceManagerImpl.broadcast " + signalSetName);
      Trace.out (">>> UserActivity.broadcast");
      Outcome outcome = _userActivity.broadcast (signalSetName);
      Trace.out ("<<< UserActivity.broadcast");
      return outcome;
      }  // end broadcast

   public ActivityToken suspend() throws NoActivityException,
      ActivityPendingException, ContextPendingException,
      NotOriginatorException, ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.suspend");
      Trace.out (">>> UserActivity.suspend");
      ActivityToken result = _userActivity.suspend ();
      Trace.out ("<<< UserActivity.suspend");
      return result;
      }  // end suspend

   public void resume (ActivityToken token) throws NoActivityException,
      ActivityPendingException, ContextPendingException, NotOriginatorException,
      SystemException, InvalidActivityException,
      ServiceNotRegisteredException, InvalidParentContextException
      {
      Trace.out ("ServiceManagerImpl.resume " + token);
      Trace.out (">>> UserActivity.resume");
      _userActivity.resume (token);
      Trace.out ("<<< UserActivity.resume");
      }  // end resume

   public ActivityToken suspendGroup() throws NoActivityException,
      ActivityPendingException, ContextPendingException,
      NotOriginatorException, ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.suspendGroup");
      Trace.out (">>> UserActivity.suspendGroup");
      ActivityToken result =  _userActivity.suspendGroup ();
      Trace.out ("<<< UserActivity.suspendGroup");
      return result;
      }  // end suspendGroup

   public void resumeGroup (ActivityToken token) throws NoActivityException,
      ActivityPendingException, ContextPendingException, NotOriginatorException,
      SystemException, InvalidActivityException,
      ServiceNotRegisteredException, InvalidParentContextException
      {
      Trace.out ("ServiceManagerImpl.resumeGroup " + token);
      Trace.out (">>> UserActivity.resumeGroup");
      _userActivity.resumeGroup (token);
      Trace.out ("<<< UserActivity.resumeGroup");
      }  // end resumeGroup

   public ActivityToken suspendAll() throws NoActivityException,
      ActivityPendingException, ContextPendingException,
      NotOriginatorException, ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.suspendAll");
      Trace.out (">>> UserActivity.suspendAll");
      ActivityToken result = _userActivity.suspendAll ();
      Trace.out ("<<< UserActivity.suspendAll");
      return result;
      }  // end suspendAll

   public void resumeAll (ActivityToken token) throws NoActivityException,
      ActivityPendingException, ContextPendingException, NotOriginatorException,
      SystemException, InvalidActivityException,
      ServiceNotRegisteredException, InvalidParentContextException
      {
      Trace.out ("ServiceManagerImpl.resumeAll " + token);
      Trace.out (">>> UserActivity.resumeAll");
      _userActivity.resumeAll (token);
      Trace.out ("<<< UserActivity.resumeAll");
      }  // end resumeAll

   public void setCompletionStatus (int completionStatus) throws
      ServiceNotRegisteredException, NoActivityException,
      SystemException, InvalidStateException
      {
      Trace.out ("ServiceManagerImpl.setCompletionStatus " + completionStatus);
      Trace.out (">>> UserActivity.setCompletionStatus");
      _userActivity.setCompletionStatus (completionStatus);
      Trace.out ("<<< UserActivity.setCompletionStatus");
      }  // end setCompletionStatus

   public int getCompletionStatus () throws
      ServiceNotRegisteredException, NoActivityException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getCompletionStatus");
      Trace.out (">>> UserActivity.getCompletionStatus");
      int compstatus = _userActivity.getCompletionStatus ();
      Trace.out ("<<< UserActivity.getCompletionStatus");
      String cs = null;
      switch (compstatus)
         {
      case CompletionStatus.CompletionStatusSuccess:
         cs = "CompletionStatusSuccess";
         break;
      case CompletionStatus.CompletionStatusFail:
         cs = "CompletionStatusFail";
         break;
      case CompletionStatus.CompletionStatusFailOnly:
         cs = "CompletionStatusFailOnly";
         break;
      default :
         cs = "" + compstatus;
         }

      _testLog.log ("getCompletionStatus returns " + cs);
      return compstatus;
      }  // end getCompletionStatus

   public int getStatus () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getStatus");
      Trace.out (">>> UserActivity.getStatus");
      int status =  _userActivity.getStatus ();
      Trace.out ("<<< UserActivity.getStatus");

      String cs = null;
      switch (status)
         {
      case Status.StatusActive:
         cs = "StatusActive";
         break;
      case Status.StatusCompleted:
         cs = "StatusCompleted";
         break;
      case Status.StatusCompleting:
         cs = "StatusCompleting";
         break;
      case Status.StatusNoActivity:
         cs = "StatusNoActivity";
         break;
      // case Status.StatusUnknown:
      //    cs = "StatusUnknown";
      //    break;
      default :
         cs = "" + status;
         }

      _testLog.log ("getStatus returns " + cs);
      return status;
      }  // end getStatus

   public void setTimeout (int timeout) throws
      ServiceNotRegisteredException, TimeoutRangeException, SystemException
      {
      Trace.out ("ServiceManagerImpl.setTimeout " + timeout);
      Trace.out (">>> UserActivity.setTimeout");
      _userActivity.setTimeout (timeout);
      Trace.out ("<<< UserActivity.setTimeout");
      }  // end setTimeout

   public int getTimeout () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getTimeout");
      Trace.out (">>> UserActivity.getTimeout");
      int compstatus = _userActivity.getTimeout ();
      Trace.out ("<<< UserActivity.getTimeout");
      return compstatus;
      }  // end getTimeout

   public String getName () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getName");
      Trace.out (">>> UserActivity.getName");
      String name = _userActivity.getName ();
      Trace.out ("<<< UserActivity.getName");
      return name;
      }  // end getName

   public GlobalId getGlobalId () throws
      ServiceNotRegisteredException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getGlobalId");
      Trace.out (">>> UserActivity.getGlobalId");
      GlobalId gid = _userActivity.getGlobalId ();
      Trace.out ("<<< UserActivity.getGlobalId");
      return gid;
      }  // end getGlobalId

   public PropertyGroup getPropertyGroup (String name) throws
      ServiceNotRegisteredException, PropertyGroupUnknownException,
      NoActivityException, SystemException
      {
      Trace.out ("ServiceManagerImpl.getPropertyGroup " + name);
      Trace.out (">>> UserActivity.getPropertyGroup");
      PropertyGroup result =  _userActivity.getPropertyGroup (name);
      Trace.out ("<<< UserActivity.getPropertyGroup");
      return result;
      }  // end getPropertyGroup

   public ActivityCoordinator getCoordinator () throws
      ServiceNotRegisteredException, SystemException, NoImplementException
      {
      Trace.out ("ServiceManagerImpl.getCoordinator");
      Trace.out (">>> UserActivity.getCoordinator");
      ActivityCoordinator result = _userActivity.getCoordinator ();
      Trace.out ("<<< UserActivity.getCoordinator");
      return result;
      }  // end getCoordinator

   public ActivityCoordinator getParentCoordinator () throws
      ServiceNotRegisteredException, SystemException, NoImplementException
      {
      Trace.out ("ServiceManagerImpl.getParentCoordinator");
      Trace.out (">>> UserActivity.getParentCoordinator");
      ActivityCoordinator result = _userActivity.getParentCoordinator ();
      Trace.out ("<<< UserActivity.getParentCoordinator");
      return result;
      }  // end getParentCoordinator

   public void addAction (Action a, String sigset, int priority)
      throws SignalSetUnknownException, SystemException, NoImplementException,
      ServiceNotRegisteredException, NoActivityException
      {
      Trace.out ("ServiceManagerImpl.addAction " + a + " SignalSet: " + sigset);
      Trace.out (">>> UserActivity.getCoordinator");
      ActivityCoordinator coord = _userActivity.getCoordinator ();
      Trace.out ("<<< UserActivity.getCoordinator");
      if (coord == null)
         throw new NoActivityException ();

      Trace.out (">>> ActivityCoordinator.addAction");
      coord.addAction (a, sigset, priority);
      Trace.out ("<<< ActivityCoordinator.addAction");

      // now add to our recovery table
      addAction (coord.getGlobalId(), sigset, new ActionPriority (a, priority));
      }  // end addAction

   public void addGlobalAction (Action a, int priority)
      throws SystemException, NoImplementException,
      ServiceNotRegisteredException, NoActivityException
      {
      Trace.out ("ServiceManagerImpl.addGlobalAction " + a);
      Trace.out (">>> UserActivity.getCoordinator");
      ActivityCoordinator coord = _userActivity.getCoordinator ();
      Trace.out ("<<< UserActivity.getCoordinator");
      if (coord == null)
         throw new NoActivityException ();

      Trace.out (">>> ActivityCoordinator.addGlobalAction");
      coord.addGlobalAction (a, priority);
      Trace.out ("<<< ActivityCoordinator.addGlobalAction");

      // now add to our recovery table
      addAction (coord.getGlobalId(), null, new ActionPriority (a, priority));
      }  // end addGlobalAction

   public void removeAction (Action a, String signalSetName)
      throws  ActionNotFoundException, SystemException,
      ServiceNotRegisteredException, NoImplementException, NoActivityException
      {
      Trace.out ("ServiceManagerImpl.removeAction " + a);
      Trace.out (">>> UserActivity.getCoordinator");
      ActivityCoordinator coord = _userActivity.getCoordinator ();
      Trace.out ("<<< UserActivity.getCoordinator");
      if (coord == null)
         throw new NoActivityException ();

      Trace.out (">>> ActivityCoordinator.removeAction");
      coord.removeAction (a, signalSetName);
      Trace.out ("<<< ActivityCoordinator.removeAction");

      // now remove from our recovery table
      deleteAction (coord.getGlobalId(), signalSetName, a);
      }  // end removeAction

   public void removeGlobalAction (Action a)
      throws  ActionNotFoundException, SystemException,
      ServiceNotRegisteredException, NoImplementException, NoActivityException
      {
      Trace.out ("ServiceManagerImpl.removeGlobalAction " + a);
      Trace.out (">>> UserActivity.getCoordinator");
      ActivityCoordinator coord = _userActivity.getCoordinator ();
      Trace.out ("<<< UserActivity.getCoordinator");
      if (coord == null)
         throw new NoActivityException ();

      Trace.out (">>> ActivityCoordinator.removeGlobalAction");
      coord.removeGlobalAction (a);
      Trace.out ("<<< ActivityCoordinator.removeGlobalAction");

      // now remove from our recovery table
      deleteAction (coord.getGlobalId(), null, a);
      }  // end removeGlobalAction



   /**
    * Queries the UserActivity instance for current ServiceManager.
    * Logs a message detailing the SM's name and context group
    *
    */
   public void getService () throws SystemException
      {
      Trace.out ("ServiceManagerImpl.getService");
      Trace.out (">>> UserActivity.getService");
      ServiceManager sm = _userActivity.getService ();
      Trace.out ("<<< UserActivity.getService");
      if (sm == null)
         _testLog.log ("getService returned null");
      else
         {
         ServiceInformation si = sm.getServiceInformation ();
         if (si == null)
            _testLog.log ("null service information");
         else
            {
            _testLog.log ("getService returned serviceName "
               + si.getServiceName ());
            _testLog.log ("getService returned contextGroup "
               + si.getContextGroup ());
            }
         }

      }  // end getService


// *********************************************
// *** ServiceManager INTERFACE METHODS ********
// *********************************************

   public SignalSet getSignalSet (String signalSetName)
      throws SignalSetUnknownException, NoImplementException
      {
      Trace.out ("ServiceManagerImpl.getSignalSet " + signalSetName);

      SubordinateSignalSet ss = (SubordinateSignalSet) _signalSets.get (signalSetName);
      if (ss != null)
         {
         _testLog.log ("Service " + _si.getServiceName () +
            " returning SignalSet " + signalSetName);
         return ss;  // ???? return ss.clone ();
         }
      else
         {
         _testLog.log ("Service " + _si.getServiceName () +
            "SignalSet " + signalSetName + " unknown");
         throw new SignalSetUnknownException ();
         }
      }  // end getSignalSet

   public SubordinateSignalSet getSubordinateSignalSet (String signalSetName)
      throws SignalSetUnknownException, NoImplementException
      {
      Trace.out ("ServiceManagerImpl.getSubordinateSignalSet " + signalSetName);
      return (SubordinateSignalSet)getSignalSet(signalSetName);
      }  // end getSignalSet

   public ServiceInformation getServiceInformation ()
      /* throws NoImplementException */
      {
      Trace.out ("ServiceManagerImpl.getServiceInformation");
      return _si;
      }  // end getServiceInformation

   public PropertyGroupManager getPropertyGroupManager (
      String PropertyGroupName)
      throws PropertyGroupUnknownException /* , NoImplementException */
      {
      Trace.out (
         "ServiceManagerImpl.getPropertyGroupManager " + PropertyGroupName);
      PropertyGroupManager pgm =
         (PropertyGroupManager) _pgms.get (PropertyGroupName);

      // let activity manager take care of this?
      if (PropertyGroupName.equals ("null"))   // special case test 261
         throw new PropertyGroupUnknownException ();

      return pgm;
      }  // end getPropertyGroupManager

   public String[] getPropertyGroupNames ()
      {
      Trace.out ("ServiceManagerImpl.getPropertyGroupNames");
      if (_pgms.isEmpty ())
         return null;
      else
         return (String[])   _pgms.keySet ().toArray (_pgNames);
      }  // end getPropertyGroupNames

   public String getCompletionSignalSetName ()
      {
      Trace.out ("ServiceManagerImpl.getCompletionSignalSetName "+_cssName);
      return _cssName;
      }  // end getCompletionSignalSetName

   public int getActivityType ()
      {
      Trace.out ("ServiceManagerImpl.getActivityType " + _type);
      return _type;
      }  // end getActivityType

   public void activityImported (ServiceInformation serviceInfo)
      {
         Trace.out ("ServiceManagerImpl.activityImported " + serviceInfo);
         _si = serviceInfo;
      }

   public SignalSet[] recoverSignalSets (GlobalId globalId)
      {
      Trace.out ("ServiceManagerImpl.recoverSignalSets");
      SignalSet[] result = null;
      // could look in recovery table ... Hashmap   ss = (HashMap) globalId.get (globalId);
      // but may as well just return complete list ...
      result = new SignalSet[_signalSets.size()];
      java.util.Enumeration enum   = _signalSets.elements ();
      int i = 0;
      while (enum.hasMoreElements ())
         {
         result [i++] = (SignalSet) enum.nextElement();
         }

      return result;
      }  // end recoverSignalSets

   public RecoverableAction[] recoverActions (GlobalId globalId, String signalSetName)
      {
      Trace.out ("ServiceManagerImpl.recoverActions " + signalSetName);
      ActionPriority[] result = null;
      HashMap   ss = (HashMap) globalIds.get (globalId);
      if (ss != null)
         {
         Vector actions = (Vector) ss.get (signalSetName);
         if (actions != null)
            {
            result = new ActionPriority[actions.size()];
            for (int i = 0; i < actions.size(); i++)
                result[i] = (ActionPriority) actions.elementAt(i);
            }

         }

      return result;
      }  // end recoverActions

   public boolean reactivate(GlobalId globalId) throws ActivityCompletedException, SystemException
     {
     Trace.out ("ServiceManagerImpl.reactivate" + globalId);
     return false; // unittest HLS does not support recovery
     } // end reactivate

   // static methods ... for dealing with recoverableActions
   public static void addAction (GlobalId gid, String signalSetName, ActionPriority ap)
      {
      Trace.out ("ServiceManagerImpl.addAction " + gid);
      HashMap   ss = (HashMap) globalIds.get (gid);
      if (ss == null)
         {
         ss = new HashMap();
         globalIds.put (gid, ss);
         }
      Vector actions = (Vector) ss.get (signalSetName);
      if (actions == null)
         {
         actions = new Vector();
         ss.put (signalSetName, actions);
         }

      actions.add (ap);
      }

   public static void deleteAction (GlobalId gid, String signalSetName, Action a)
      {
      Trace.out ("ServiceManagerImpl.deleteAction " + gid);

      HashMap   ss = (HashMap) globalIds.get (gid);
      if (ss != null)
         {
         Vector actions = (Vector) ss.get (signalSetName);
         if (actions != null)
            {
            for (int i = actions.size()-1; i >= 0; i--)
               {
               ActionPriority ap = (ActionPriority) actions.elementAt(i);
               if (ap.getAction() == a)
                  {
                  actions.remove (i);
                  break;
                  }
               }
            }
         }
      }


   // Private attributes
   private String        _cssName        = null;
   private Hashtable     _signalSets     = new Hashtable ();
   private Hashtable     _pgms           = new Hashtable ();
   private ServiceInformation _si        = null;
   private ActivityManager _userActivity = null;

   private String[] _pgNames = new String [1];
   private int _type         = 0x4A324545;

   private TestLog       _testLog = null;

   // private static attributes
   // a hashtable indexed by globalid containing hashmaps indexed by signalsetname which
   // contain vectors of recoverableActions
   private static Hashtable     globalIds = new Hashtable ();    // save recoverable actions

   private class ActionPriority implements javax.activity.coordination.RecoverableAction
      {
      public ActionPriority (Action a, int p)
         {
         _action = a;
         _priority = p;
         }

      public Action getAction()
         {
         return _action;
         }

      public int getPriority()
         {
         return _priority;
         }

      private Action _action;
      private int    _priority;
      }
   }  // end class ServiceManagerImpl
