/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/* Implementation of the javax.activity.coordination.SignalSet        */
/* interface. A SignalSet is a producer of Signals and a consumer of  */
/* Action Outcomes. It produces a final Outcome at the end of         */
/* Signal processing.                                                 */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.hls;

import javax.activity.*;
import javax.activity.coordination.*;

import java.util.Vector;
import java.util.Hashtable;


/**
 * An implementation of the <code>SignalSet</code> interface.
 * A SignalSet is a producer of Signals and a consumer of
 * <code>Action</code> <code>Outcome</code>s. It produces a final
 * Outcome at the end of Signal processing.
 * This class provides a configurable set of <code>Signal</code>s
 * and configurable responses to <code>Outcome</code>s.
 */

public final class SignalSetImpl implements SubordinateSignalSet
   {
   /**
    * Construct a SignalSet with the given name.
    * @param ssName   the name of the <code>SignalSet</code>.
    * @param testLog  object used for logging test status
    */
   public SignalSetImpl (String ssName, TestLog testLog)
      {
      Trace.out ("SignalSetImpl constructor for " + ssName);
      _ssName  = ssName;
      _testLog = testLog;
      }  // end c'tor

   /**
    * Add the named <code>Signal</code> to the list of Signals this
    * SignalSet produces.
    * Signals are produced in the order that they are added.
    * @param signalName   the name of the <code>Signal</code> to be added.
    */
   public void addSignal (String signalName)
      {
      Trace.out ("SignalSetImpl.addSignal for " + signalName);
      Signal sig = new Signal (signalName, _ssName, (java.io.Serializable) null);
      _signals.add (sig);
      }  // end addSignal

   public void addResponse (String respName, boolean isActionInterested,
      boolean useNextSignal)
      {
      Trace.out ("SignalSetImpl.addResponse for " + respName);
      CoordinationInformation sigl = new CoordinationInformation (isActionInterested, useNextSignal);
      _responses.put (respName, sigl);
      }  // end addResponse

   public void addOutcome (String outcome)
      {
      Trace.out ("SignalSetImpl.addOutcome " + outcome);
      _outcome = new Outcome (outcome, (java.io.Serializable) null);
      }  // end addOutcome

   // INTERFACE METHODS

   public String getSignalSetName ()
      {
      Trace.out ("SignalSetImpl.getSignalSetName " + _ssName);
      return _ssName;
      }  // end getSignalSetName

   public Signal getSignal ()
      {
      Trace.out ("SignalSetImpl.getSignal");
      Signal sig     = null;
      String signame = null;

      if (_sigindex < _signals.size ())
         {
         sig     = (Signal) _signals.elementAt (_sigindex);
         signame = sig.getName ();
         _sigindex++;
         }

      Trace.out (" *** returning " + signame);
      if (_responses.get ("getsignal") != null)
         _testLog.log ("SignalSet " + _ssName + " returning Signal " + signame);
      return sig;
      }  // end getSignal

   public Outcome getOutcome () throws SignalSetActiveException
      {
      Trace.out ("SignalSetImpl.getOutcome " + _outcome);
      // if (_signindex < _signals.size()) throw new SignalSetActiveException();
      // this is also the signal to reset
      // _signindex = 0;
      return _outcome;
      }  // end getOutcome

   public void setCompletionStatus (int completionStatus, int status)
      /* throws SignalSetActiveException */
      {
      Trace.out ("SignalSetImpl.setCompletionStatus " + completionStatus);
      _cs       = completionStatus;
      _status   = status;

      if (_responses.get ("setcompletionstatus") != null)
        {
         String cs = null;
         switch (_cs)
            {
         case CompletionStatus.CompletionStatusSuccess:
            cs = "CompletionStatusSuccess";
            break;
         case CompletionStatus.CompletionStatusFail:
            cs = "CompletionStatusFail";
            break;
         case CompletionStatus.CompletionStatusFailOnly:
            cs = "CompletionStatusFailOnly";
            break;
         default :
            cs = "" + _cs;
         }
         _testLog.log (
            "SignalSet " + _ssName + " received completionStatus " + cs);
        }  // end if

      // Setting the CompletionStatus will RESET signal generation
      _sigindex = 0;   // reset signal processing
      }  // end setCompletionStatus

   public int getCompletionStatus () throws SignalSetActiveException
      {
      Trace.out ("SignalSetImpl.getCompletionStatus " + _cs);
      return _cs;
      }  // end getCompletionStatus

   public void setSignal (Signal signal)
      {
      Trace.out ("SignalSetImpl.setSignal " + signal);
      // _signal = signal;
      }

   public CoordinationInformation setResponse (Outcome response)
      throws SignalSetInactiveException
      {
      Trace.out ("SignalSetImpl.setResponse " + response);
      CoordinationInformation signaling = null;

      // if no Signals have been requested, then there can be no response
      // so should probably throw SignalSetInactiveException

      String outcome = null;
      String extdata = null;
      if (response != null)
         {
         outcome = response.getName ();
         if (outcome != null &&
            outcome.equals ("org.omg.CosActivity.ActionError"))
            _cs = CompletionStatus.CompletionStatusFailOnly;

         signaling = (CoordinationInformation) _responses.get (outcome);
         try {
            java.io.Serializable ioser = response.getExtendedValue ();
            if (ioser != null)
               extdata = ioser.toString ();
             }
         catch (Exception e)
            {
      Trace.out ("SignalSetImpl.setResponse " + e);
            }

         }  // end if

      if (signaling == null)
         signaling = _defaultSignaling;

      _testLog.log ("SignalSet " + _ssName + " received response " + outcome);
      if (extdata != null)
        _testLog.log ("SignalSet " + _ssName + " received extdata " + extdata);

      Trace.out ("SignalSetImpl.setResponse received " + outcome);
      return signaling;
      }  // end setResponse

   public Outcome getCurrentOutcome ()
      {
      Trace.out ("SignalSetImpl.getCurrentOutcome " + _outcome);
      return _outcome;
      }  // end getCurrentOutcome

   public void destroy ()
      {
      Trace.out ("SignalSetImpl.destroy");

      if (_responses.get ("destroy") != null)
         _testLog.log ("SignalSet " + _ssName + " destroyed");

      }  // end destroy

   public void setActivityCoordinator (ActivityCoordinator coord)
      throws SignalSetActiveException
      {
      Trace.out ("SignalSetImpl.setActivityCoordinator");
      }  // end setActivityCoordinator


   // Private attributes
   private Outcome    _outcome   = null;
   private String     _ssName    = null;
   private int        _sigindex  = 0;
   /**
    * default <code>CoordinationInformation</code> to be returned during setResponse
    * when the <code>Outcome</code> is not recognised.
    */
   private CoordinationInformation  _defaultSignaling = new CoordinationInformation (true, false);

   /**
    * list of <code>Signal</code>s to be produced.
    */
   private Vector     _signals   = new Vector ();

   /**
    * table of <code>CoordinationInformation</code> objects to be returned by setResponse
    * keyed by the <code>Outcome</code> name.
    */
   private Hashtable  _responses = new Hashtable ();

   private int          _cs = 0;
   private int      _status = 0;
   private TestLog _testLog = null;

   }  // end class SignalSetImpl
