/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION: An implementation of the Action interface to allow    */
/* testing of the interaction with the Activity Service.              */
/* Each Action object contains a TestLog reference which is used to   */
/* log the interactions as appropriate for verification of correct    */
/* functioning.                                                       */
/**********************************************************************/

package com.ibm.activity.hls;

import javax.activity.*;
import javax.activity.coordination.*;

import java.util.Hashtable;


/**
 * An implementation of the <code>Action</code> interface.
 * This class is configurable in the <code>Outcome</code> that it returns
 * in response to a given <code>Signal</code>.
 */

public final class TestAction implements Action, Comparable
   {
   /**
    * Create an Action with the given name
    * @param name     the name of the <code>Action</code> being created
    * @param testLog  the TestLog object used to log information
    */
   public TestAction (java.lang.String name, TestLog testLog)
      {
      Trace.out ("TestAction.TestAction " + name);
      _name    = name;
      _testLog = testLog;
      }  // end c'tor


   /**
    * add a configured response for the given signal.
    * @param signalname    the name of the <code>Signal</code> that this
    *     response should be generated for.
    * @param outcome       the name of the <code>Outcome</code> that is
    *     generated
    */
   public void addResponse (String signalname, String outcome)
      {
      Trace.out (
         "TestAction.addResponse " + signalname + " Outcome " + outcome);
      _responses.put (signalname, outcome);
      }  // end addResponse


   /**
    * add a configured command for the given signal.
    * @param signalname    the name of the <code>Signal</code> that this
    *     response should be generated for.
    * @param cmds          the Command object to run for this Signal
    */
   public void addCommand (String signalname, Command cmds)
      {
      Trace.out ("TestAction.addCommand " + signalname);
      _commands.put (signalname, cmds);
      }  // end addCommand


   /**
    * Getter for the GlobalId which is extracted from each signal
    */
   public GlobalId getGlobalId ()
      {
      Trace.out ("TestAction.getGlobalId " + _gid);
      return _gid;
      }  // end getGlobalId


   /**
    * Getter for the CompletionStatus which is extracted from each signal
    */
   public int  getCompletionStatus ()
      {
      Trace.out ("TestAction.getCompletionStatus " + _completionStatus);
      return _completionStatus;
      }  // end getCompletionStatus


   /**
    * Getter for the finalOutcome which is extracted from each signal
    */
   public Outcome  getFinalOutcome ()
      {
      Trace.out ("TestAction.getFinalOutcome " + _finalOutcome);
      return _finalOutcome;
      }  // end getFinalOutcome


   // Comparable INTERFACE METHODS

   public int compareTo (Object o)
      {
      return _name.compareTo (((TestAction) o)._name);
      }

   public String toString()
      {
      return _name;
      }

   // Action INTERFACE METHODS

   public Outcome processSignal(Signal signal) throws ActionErrorException
      {
      Trace.out ("TestAction.processSignal " + signal);
      Outcome result = null;
      String signalname = signal.getName ();
      String sigsetname = signal.getSignalSetName ();

      _testLog.log ("Action " + _name + " Received " +
         signalname + " from " + sigsetname);

      // try {
         // extract extended info from Signal
         java.io.Serializable ioser = signal.getExtendedValue ();
         if (ioser != null && (ioser instanceof ActivityInformation))
            {
            Trace.out ("TestAction.processSignal: found extended data");
            _gid = ((ActivityInformation) ioser).getGlobalId ();
            _completionStatus =
               ((ActivityInformation) ioser).getCompletionStatus ();
            _finalOutcome     =
               ((ActivityInformation) ioser).getFinalOutcome ();
            }
         // else set fields to null ????
      //    }
      // catch (SystemException e)
      //    {
      //    Trace.out (
      //       "TestAction.processSignal: error extracting extended data " + e);
      //    }

      String outcome = (String) _responses.get (signalname);
      if (outcome != null)
         {
         if (outcome.equals ("actionerror"))
            {
            // _testLog.log ("Action " + _name + " throwing ActionErrorException");
            Trace.out ("TestAction.processSignal: Action " +
               _name + " throwing ActionErrorException");
            throw new ActionErrorException ();
            }
         else if (outcome.equals ("actionsystemexception"))
            {
            // _testLog.log ("Action " + _name + " throwing Exception");
            Trace.out ("TestAction.processSignal: Action " +
               _name + " throwing Exception");
            throw new NullPointerException (signalname);
            }

         result = new Outcome (outcome, (java.io.Serializable) null);
         Trace.out (" *** returning outcome " + outcome);
         _testLog.log ("Action " + _name + " returning " + outcome);
         }

      Command cmd = (Command) _commands.get (signalname);
      if (cmd != null)
         {
         Trace.out ("TestAction.processSignal: executing Command object");
         cmd.runCommands ();
         }

      return result;
      }

   public void destroy()
      {
      Trace.out ("TestAction.destroy");

      // slight hack to turn on logging for Actions ...
      String outcome = (String) _responses.get ("destroy");
      if (outcome != null)
         _testLog.log ("Action " + _name + " destroyed");
      }

   // private member variables
   private Hashtable   _responses = new Hashtable ();
   private Hashtable   _commands  = new Hashtable ();
   private java.lang.String _name = null;
   private TestLog       _testLog = null;
   private GlobalId          _gid = null;          // extract from Signals
   private int  _completionStatus = 0;             // extract from Signals
   private Outcome  _finalOutcome = null;          // extract from Signals

   }  // end class TestAction
