/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION: Controls the execution of a series of tests.          */
/* Test descriptions are read from a file and either those that are   */
/* specified on the command line are run or if none are specified,    */
/* then all tests are run.                                            */
/* Each test is executed in its own process (jvm) to prevent any      */
/* interaction between tests.                                         */
/**********************************************************************/

package com.ibm.activity.unittest;

import java.io.*;
import java.lang.reflect.*;



/**
 * A program to execute a series of tests given a file containing the
 * test description.
 *  The format of this file is a series of lines of the form
 * <br>
 * <i>testname classname arguments</i>
 * <br> where classname is the java test class containing the
 * <code>runTest</code> method and arguments are the
 * command line arguments that this class is expecting in the first
 * parameter of the <code>runTest</code> method.
 */

public class ASTest
   {

   /**
    * Execute a series of tests based on test description file.
    * After test execution, a summary file (in html format) is created
    * giving a summary of tests run, result and date last executed.
    *
    * @param  args   the command line arguments for this execution.
    * <br> the first argument must be the name of the test description file.
    * Subsequent arguments are interpreted as specific test identifiers to
    * be executed.
    */
   public static void main (String args[])
      {
      try {
         // extract parameters java ASTest <testfile>
         String testfile = null;
         if (args.length > 0)
            testfile = args [0];
         else
            throw new Exception ("no file specified");

         // read imput file and store test descriptions in table
         File f = new File (args [0]);
         BufferedReader rdr = new BufferedReader (
            new InputStreamReader (new FileInputStream (f)));
         String testline = null;
         java.util.Hashtable table = new java.util.Hashtable ();
         java.util.Vector    list  = new java.util.Vector ();
         while ((testline = rdr.readLine ()) != null)
            {
            testline = testline.trim ();
            if (testline.length () > 0)
               {
               String s = testline.substring (0, testline.indexOf (" "));
               if (s != null)
                  {
                  table.put (s.toLowerCase (), testline);
                  list.add (s);
                  }

               }  // end if

            }  // end while

         // extract properties (cur dir etc)
         String sep    = System.getProperty ("file.separator");
         String jhome  = System.getProperty ("java.home");
         String jclass = System.getProperty ("java.class.path");
         String pwd    = System.getProperty ("user.dir", ".");
         Runtime rt    = Runtime.getRuntime ();

         // now select which tests to run ... from args
         int passed = 0, failed = 0, errors = 0, total = 0;

         if (args.length > 1)
            {
            for (int i = 1; i < args.length; i++)
               {
               String newargs = (String) table.get (args [i].toLowerCase ());
               if (newargs != null)
                  {
                  int result = execTest (rt, newargs);
                  total++;
                  switch (result)
                     {
                  case 1: passed++; break;
                  case 2: failed++; break;
                  case 3: errors++; break;
                     }
                  }  // end if
               else
                  {
                  System.out.println ("Test " + args[i] + " not defined in " + args[0]);
                  }

               }  // end for

            }
         else   // run ALL tests
            {
            java.util.Enumeration enum = table.elements ();
            while (enum.hasMoreElements ())
               {
               // build argument list for RunTest jvm
               String newargs = (String) enum.nextElement ();
               int result = execTest (rt, newargs);
               total++;
               switch (result)
                  {
               case 1: passed++; break;
               case 2: failed++; break;
               case 3: errors++; break;
                  }
               // System.out.println ("Exit value = " + p.exitValue ());
               } // ene while

            }  // end else

         // printout some stats on tests performed?
         // don't know which passed/failed!
         System.out.println ("SUMMARY");
         System.out.println ("Total tests executed: " + total);
         System.out.println ("      tests passed  : " + passed);
         System.out.println ("      tests failed  : " + failed);
         System.out.println ("      test  errors  : " + errors);

         // now run summary to update summary file
         File   resultdir = new File (pwd, "results");
         if (resultdir.exists ())
            {
            // open results.htm file
            File   resfile = new File (pwd, "summary.htm");
            PrintWriter resstream = new PrintWriter (
               new FileOutputStream (resfile), true);

            // write some header html to result.htm
            resstream.println ("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">");
            resstream.println ("<HTML>");
            resstream.println ("<HEAD>");
            resstream.println ("<TITLE>JSR95 TCK Test Results");
            resstream.println ("</TITLE>");
            resstream.println ("</HEAD>");
            resstream.println ("<BODY>");
            // table headers
            resstream.println ("<TABLE BORDER=1 CLASS=TestDescription>");
            resstream.println ("<TR>");
            resstream.println ("<TH>test case</TH>");
            resstream.println ("<TH>result</TH>");
            resstream.println ("<TH>date last run</TH>");
            resstream.println ("<TH>failure message</TH>");
            resstream.println ("</TR>");

            // read each file in results dir and store in testname order
            for (int i = 0; i < list.size (); i++)
               {
               String testname = (String) list.elementAt (i);
               String testdate = null;
               String outcome  = null;
               String info     = null;

               File fresult = new File (resultdir, (String) list.elementAt (i));
               if (fresult.exists ())
                  {
                  // create a reader and readLine
                  BufferedReader filrdr = new BufferedReader (
                     new InputStreamReader (new FileInputStream (fresult)));
                  String data = filrdr.readLine ();
                  if (data != null && data.length () > 5)
                     {
                     // parse the testname, pass or fail
                     int index = data.indexOf ("Test:");
                     if (index > 0)
                        {
                        // write out suitable html to results.htm file
                        testdate = data.substring (5, index);
                        testname = data.substring (index+5, index+16).trim ();

                        outcome = data.substring (index+16);
                        int infoindex = outcome.indexOf (":");
                        if (infoindex > 0)
                           {
                           info = outcome.substring (infoindex + 1);
                           outcome = outcome.substring (0, infoindex);
                           }
                        else
                           {
                           info = "";
                           }

                        // resstream.println (testname + " | " + outcome + " |" +
                        //    testdate + " | " + info);
                       }  // end if

                     }  // end if

                  } // end if
               else
                  {
                  // resstream.println (testname + " | " + "Not Tested" + " |" +
                  //          "                   " + " | ");
                  testdate = "not tested";
                  outcome  = "n/a";
                  info     = "";
                  }

               resstream.println ("<TR>");
               resstream.println ("<A NAME=" + testname +">");
               resstream.println ("<TD>" + testname + "</TD>");
               resstream.println ("<TD>" + outcome + "</TD>");
               resstream.println ("<TD>" + testdate + "</TD>");
               resstream.println ("<TD>" + info + "</TD>");
               resstream.println ("</TR>");

               }  // end for

            // write end table tags
            resstream.println ("</TABLE>");
            // write end html tags
            resstream.println ("</BODY>");
            resstream.println ("</HTML>");

            System.out.println ("finished");
            }  // end if

         }
      catch (Exception e)
         {
         System.out.println ("Syntax: java ASTest <name> <class> <args ...>");
         System.out.println ("Exception: " + e);
         e.printStackTrace ();
         }

      }  // end main

   /**
    * Execute a single test in a separate process.
    * The stdout and stderr streams of the child process are read and
    * echoed to System.out.
    * The child process controls the logging of test output and only
    * returns a summary of test result which is interpreted by this
    * method.
    * @param  rt         the java.Runtime object used to spawn child process.
    * @param  cmdstring  the child process command to be executed.
    * @return integer representing the result of the test execution
    *    (PASSED, FAILED or ERROR).
    */
   private static int execTest (Runtime rt, String cmdstring)
      throws Exception
      {
      String sep    = System.getProperty ("file.separator");
      String testarg = "." + sep + "testone.bat " + cmdstring;
      int result = 0;

      // now run the test ...
      // Process p = rt.exec ("d:/jdk1.3/bin/java -fullversion");
      // Process p = rt.exec ("java 'com.ibm.activity.unittest.RunTest' test1 'com.ibm.activity.unittest.TestActivity' 'testcase/jsr9500003.tst'");
      // Process p = rt.exec ("d:/jdk1.3/bin/java -classpath " + jclass + " com.ibm.activity.unittest.RunTest test1 com.ibm.activity.unittest.TestActivity testcase/jsr9500003.tst", null);
      // Process p = rt.exec ("./run.bat test1 com.ibm.activity.unittest.TestActivity testcase\\jsr9500003.tst", null);
      Process p = rt.exec (testarg, null);

      String line = null;
      BufferedReader outrdr = new BufferedReader (
         new InputStreamReader (p.getInputStream ()));
      while ((line = outrdr.readLine ()) != null)
         {
         // look for PASSED. FAILED or ERROR and return suitable int
         if (line.indexOf ("PASSED:") != -1)
            result = 1;
         if (line.indexOf ("FAILED:") != -1)
            result = 2;
         if (line.indexOf ("ERROR:") != -1)
            result = 3;
         System.out.println (line);
         }

      BufferedReader errrdr = new BufferedReader (
         new InputStreamReader (p.getErrorStream ()));
      while ((line = errrdr.readLine ()) != null)
         System.out.println (line);

      p.waitFor ();
      return result;
      }  // end execTest

   }  // end class ASTest
