/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION: A utility class to control the execution of a single  */
/* test, directing the test output to appropriate files and recording */
/* the overall test result.                                           */
/* The test output is stored in the 'logs' directory under the names  */
/* <testname>.out and <testname>.err.  The overall result is stored   */
/* in the 'results' directory in file <testname>                      */
/**********************************************************************/

package com.ibm.activity.unittest;

import java.io.*;
import java.lang.reflect.*;
// import java.util.*;



/**
 * This class is used to control the execution of a single test.
 * The output (stdout and stderr) are captured and redirected to
 * suitable log files, while the overall test result, date and testname
 * are written to a results file.
 */

public class RunTest
   {

   /**
    * Provides the ability to run the test standalone or in a suitable
    * test harness.
    * @param  args   the arguments to this test execution.
    *   These are
    * <OL>
    * <LI> The test name.  This determines the log and result file names.
    * <LI> The java class to run.  Introspection is used to create an
    *      object of this class and to call its 'runTest' method passing
    *      appropriate parameters and interpreting the result.
    * <LI> Arguments to the test.  Any number of arguments, but generally
    *      the location of the test command script that will be interpreted
    *      by the execution.
    * </OL>
    */
   public static void main (String args[])
      {
      String testname = null;
      String sep      = null;
      String pwd      = null;
      Object result   = null;

      PrintWriter errstream = null;
      PrintWriter outstream = null;

      try {
         // extract parameters java RunTest <name> <class> <args>
         if (args.length > 0)
            testname = args [0];
         else
            throw new Exception ();

         String testclass = null;
         if (args.length > 1)
            testclass = args [1];
         else
            throw new Exception ();

         // build argument list for runTest method
         String []  newargs = new String [args.length - 2];
         for (int i = 0; i < newargs.length; i++)
            newargs[i] = args [i+2];

         // extract properties (cur dir etc)
         sep = System.getProperty ("file.separator");
         pwd = System.getProperty ("user.dir", ".");

         // open files for stdout/stderr
         File   logdir    = new File (pwd, "logs");
         if (! logdir.exists ())
            {
            if (! logdir.mkdir ())
               throw new Exception ("Failed to create logdir: " + logdir);
            }  // end if

         File   stdout = new File (logdir, testname + ".out");
         File   stderr = new File (logdir, testname + ".err");
         outstream = new PrintWriter (new FileOutputStream (stdout), true);
         errstream = new PrintWriter (new FileOutputStream (stderr), true);

         // classForName
         Class c = Class.forName (testclass);
         if (c != null)
            {
            // get runTest method via introspection
            Class [] carray = new Class [3];
            carray [0] = args.getClass ();
            carray [1] = carray [2] = outstream.getClass ();

            Method m = c.getMethod ("runTest", carray);
            Object o = c.newInstance ();

            // execute runTest & convert result to string
            Object [] oarray = new Object [3];
            oarray [0] = newargs;
            oarray [1] = errstream;
            oarray [2] = outstream;
            result = m.invoke (o, oarray);

            }  // end if

         }
      catch (Exception e)
         {
         System.out.println ("Syntax: java RunTest <name> <class> <args ...>");
         System.out.println ("Exception: " + e);
         result = new String ("Test ERROR: " + e);
         if (outstream != null)
            e.printStackTrace (outstream);
         }

      // now output the results of this test
      try {
         if (testname != null)
            {
            // parse result string
            String s = result.toString ();
            System.out.println (testname + " " + s);

            File   resultdir = new File (pwd, "results");
            if (! resultdir.exists ())
               {
               if (! resultdir.mkdir ())
                  throw new Exception (
                     "Failed to create resultdir: " + resultdir);
               }  // end if

            String date = java.text.DateFormat.getDateTimeInstance ()
               .format (new java.util.Date ());
            s = "Date: " + date + " Test: " + testname + s.substring (4);

            // open result file and write parsed data
            File resfile = new File (resultdir, testname);
            PrintWriter resstream = new PrintWriter (
               new FileOutputStream (resfile), true);
            resstream.println (s);
            }  // end if

         }
      catch (Exception e)
         {
         System.out.println ("Exception: " + e);
         }

      System.exit (0);
      }  // end main

   }  // end class RunTest
