/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION: This class provides the front end command interpreter */
/* which is used to construct and configure the various objects       */
/* needed to test the functionality of the Activity Service.          */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.unittest;

import javax.activity.*;
import javax.activity.coordination.*;
import javax.activity.propertygroup.*;
import com.ibm.activity.hls.*;

import java.io.*;
import java.util.*;



/**
 * A general purpose test interpretation class.
 * This class can be used to read and interpret a set of commands which
 * exercise the Activity Service implementation.  Results can be logged
 * and test outcome evaluated to determine compliance with the test
 * specification.
 * Commands can be entered interactively or via text files providing
 * the ability to be integrated into a suitable test harness (for
 * example the javaTest environment).
 * This class provides the capability to exercise the methods of the
 * main interface classes of the Activity Service, that is
 * <code>UserActivity</code>, <code>ActivityManager</code>
 * and <code>ActivityCoordinator</code>
 *
 * Currently this class does not provide the capability to test the
 * distributed functionality of the Activity Service.
 */

public class TestActivity extends Thread
   {
   /**
    * Provides the ability to run the test standalone or in a suitable
    * test harness.
    * Testing is accomplished by creating an instance of the class
    * and calling the runTest method.
    * This is done here in 'main', but may also be performed from other
    * methods, such as the 'run' method specified for the javaTest
    * harness.
    */
   public static void main (String args[])
      {
      Trace.out ("***Start TestActivity unit test***");

      TestActivity   test = new TestActivity ();
      TestResult result = test.runTest (args,
         new PrintWriter (System.err, true),
         new PrintWriter (System.out, true));

      System.out.println (result);

      Trace.out ("***End TestActivity unit test***");
      System.exit (0);  // to ensure any Activity Service threads are killed
      }  // end main


   /**
    * This is the entry point to begin testing from the javaTest harness.
    * See javaTest documentation for details.
    * Un-comment this method and use the correct return type.
    */
 /*
   public jsvaTest.Status run (String [] args, PrintWriter log,
      PrintWriter ref)
      {
      TestResult t  = runTest (args, log, ref);
      String result = t.toString ();
      int index;
      if ((index = result.indexOf ("PASSED:")) > 0)
         return javaTest.Status.passed (result.substring (index + 7));
      else if ((index = result.indexOf ("FAILED:")) > 0)
         return javaTest.Status.failed (result.substring (index + 7));
      else if ((index = result.indexOf ("ERROR:")) > 0)
         return javaTest.Status.error (result.substring (index + 6));
      else
         return javaTest.Status.error (result);
      }
 */


   /**
    * This is the entry point to begin testing.
    * It is called from <i>main</i> for standalone testing and
    * from the <i>run</i> method when used in the javaTest harness.
    *
    * @param args   arg[0] should be the name of a file containing
    *    the commands to be executed
    * @param log    stream into which trace information is written
    * @param ref    stream into which exception text may be written
    * @return       TestResult object which encapsulates the result
    *    of running the commands
    */
   public TestResult runTest (String [] args, PrintWriter log,
      PrintWriter ref)
      {
      // set Trace destination ...
      _ref = ref;  _log = log;
      Trace.setDestination (log);

      // get trace properties
      String traceflag = System.getProperty ("com.ibm.activity.unittest.Trace");
      if (traceflag != null)
         Trace.on ();

      // look for input file
      InputStream i = null;
      if (args != null && args.length > 0)
         {
         File f = new File (args [0]);
         try {
            // if (f.exists ())
            i = new FileInputStream (f);
            }
         catch (java.io.FileNotFoundException e)
            {
            log.println ("File " + args [0] + " not found");
            _testResult.error ("" + e);
            return _testResult;
            }
         }
      else
         i = System.in;

      // initialise the Static (class) member which provides the
      // test commands ...
      rdr = new StreamTokenizer (new BufferedReader (
         new InputStreamReader (i /*System.in*/)));
      rdr.eolIsSignificant (true);
      rdr.lowerCaseMode (true);
      rdr.slashSlashComments (true);
      rdr.slashStarComments (true);

      tbl.put ("threadMain", this);
      threads.add (this);
      release ();
      run ();
      return _testResult;
      }


   /**
    * TestActivity is  sub-class of Thread.
    * The Activity Service includes functions which need to be tested from
    * multiple threads and so test commands can create multiple instances
    * of this class, each to run in its own thread.
    * <P>
    * The threading is synchronized such that only one thread is reading
    * and executing commands at any given time, so all instances share
    * a common command stream.
    */
   public TestActivity ()
      {
      super ();
      if (_ref == null)
         _ref = new PrintWriter (System.out, true);

      if (_log == null)
         _log = new PrintWriter (System.err, true);
      }  // end c'tor

   /**
    * Read a line from the input stream and parse it into the appropriate
    * series of tokens.
    * @return   a Vector of Strings representing the parts of the command
    */
   public Vector readLine ()
      {
      Trace.out ("TestActivity.readLine");
      tokens.clear ();
      int token = StreamTokenizer.TT_EOF;
      try
         {
         token = rdr.nextToken ();
         while (token != StreamTokenizer.TT_EOL   &&
            token != StreamTokenizer.TT_EOF )
            {
            // if (token == StreamTokenizer.TT_WORD)
            if (token == StreamTokenizer.TT_NUMBER)
               tokens.add ("" + (int) rdr.nval);
            else if (rdr.sval != null)
               tokens.add (rdr.sval);

            token = rdr.nextToken ();
            }  // end while
         }
      catch (Exception e) {}

      if (token == StreamTokenizer.TT_EOF)
         return null;
      else
         return tokens;
      }  // end readLine


   /**
    * Returns the current line number in the input stream.
    * For diagnostic purposes.
    * Actually, it returns the line number AFTER the line just
    * processed :-0
    */
   public int lineno ()
      {
      Trace.out ("TestActivity.lineno");
      return (rdr == null) ? 0 : rdr.lineno ();
      }

   /**
    * Store an object in a global table using the given key.
    * Objects can be stored and retrieved by any thread.
    * tbl is a Static (class) variable and so is available to any
    * instance/thread.
    * <P>
    * The various objects (SignalSets, Actions etc) are all manipulated
    * by name using the interpreted commands.  This table is used to
    * translate from the name to the object.
    */
   public void put (String key, Object obj)
      {
      Trace.out ("TestActivity.put " + key);
      tbl.put (key, obj);
      }

   /**
    * Retrieve an object from the global table.
    */
   public Object get (String key)
      {
      Trace.out ("TestActivity.get " + key);
      return tbl.get (key);
      }


// ****************************************
// *** COMMAND INTERPRETATION ROUTINES  ***
// ****************************************

   /**
    * This function terminates all threads and returns control to the
    * main thread in runTest.
    * It is called when the 'end' command is read or when the input
    * stream reaches end of file.
    */
   public void terminate ()
      {
      Trace.out ("TestActivity.terminate");
      // end of test ... stop all threads
      for (int i = 0; i < threads.size (); i++)
         {
         TestActivity thrd = (TestActivity) threads.elementAt (i);
         thrd.finished ();
         thrd.release ();
         }  // end for

      }  // end terminate


   /**
    * Switch control to a different thread.
    * Only one thread is active (reading and executing commands) at a time.
    * If the named thread does not exist in the global table, a
    * new thread is created.
    * The initial thread is stored under the name 'threadMain'.
    * @param threadName  is the name that the thread object is stored under
    *    in the global table.
    * @return            new or existing thread object
    */
   public Thread selectThread (String threadName)
      throws Exception
      {
      Trace.out ("TestActivity.selectThread");
      Object o = get (threadName);
      if (o == null)
         {
         // not yet exist, create ..
         o = new TestActivity ();
         threads.add (o);
         put (threadName, o);             // store it
         ((TestActivity) o).start ();    // set it going
         }
      else if (!(o instanceof TestActivity))
         {
         // test case syntax error
         // throw an exception
         throw new SyntaxError (threadName + " is not valid");
         }

      // return (TestActivity) o;
      return (Thread) o;
      }  // end selectThread


   /**
    * Switch to a different ServiceManager object.
    * Most commands operate on the 'current' ServiceManager which is
    * stored in the Static (class) member 'sm1'.
    * If the named object does not exist in the global table, then
    * a new ServiceManager is created.
    * <br>
    * For a new ServiceManager object, the command syntax is
    * 'servicemanager name {contextGroup} {serviceName}'
    * The serviceName defaults to 'name'
    * <P>
    * If the ServiceManager is created with a contextGroup, then it
    * attempts to register with the Activity Service.
    *
    * @param  parms is the list of parsed tokens from the current command line
    */
   public void selectServiceMgr (Vector parms) throws Exception
      {
      Trace.out ("TestActivity.selectServiceMgr");
      // servicemanager name contextGroup serviceName
      if (parms.size () == 0)
         throw new SyntaxError ("specify service manager name");

      String svcName = (String) parms.remove (0);
      Object o = get (svcName);
      if (o != null)
         {
         // existing service
         if (! (o instanceof ServiceManagerImpl))
            throw new SyntaxError (svcName + " is not a service manager");
         else
            sm1 = (ServiceManagerImpl) o;

         // if any more parms, then try to set new context group?
         // ... which could try to re-register? (cmd line flag)
         String contextGroup = "";
         if (parms.size () > 0)
            {
            contextGroup = (String) parms.remove (0);
            boolean register = false;
            if (parms.size () > 0 &&
               ( (String) parms.remove (0)).equals ("true") )
               register = true;

            sm1.setContextGroup (svcName, contextGroup, register);
            }  // end if

         }
      else
         {
         // new service manager ...
         // if no parms, then don't register sm (use diff c'tor)
         String contextGroup = "";
         if (parms.size () > 0)
            {
            contextGroup = (String) parms.remove (0);

            String serviceName = svcName;
            if (parms.size () > 0)
               serviceName = (String) parms.remove (0);

            sm1 = new ServiceManagerImpl (serviceName, contextGroup, _testResult);
            }
         else
            sm1 = new ServiceManagerImpl (svcName, _testResult);

         put (svcName, sm1);
         }  // end else

      }  // end selectServiceMgr


   /**
    * Utility method to compare two objects (Strings or GlobalIds)
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'isequal obj1 obj2 '
    * <br>
    */
   protected void do_isequal (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_isequal");
      // isequal obj1 obj2
      if (line.size () < 2)
         throw new SyntaxError ("specify two objects for comparison");

      String name1 = (String) line.remove (0);
      Object o1 = get (name1);
      if (o1 == null)
         throw new SyntaxError (name1 + " is not valid");

      String name2 = (String) line.remove (0);
      Object o2 = null;
      if (!name2.equals ("null"))
         {
         o2 = get (name2);
         if (o2 == null)
            throw new SyntaxError (name2 + " is not valid");
         }  // end if

      boolean status = false;
      if ( (o1 instanceof GlobalId) && (o2 == null || o2 instanceof GlobalId))
         {
         Trace.out (">>> GlobalId.isEqual");
         status = ((GlobalId) o1).equals((GlobalId) o2);
         Trace.out ("<<< GlobalId.isEqual");
         }
      else if ( (o1 instanceof String) && (o2 instanceof String))
         status = ((String) o1).equals ((String) o2);
      else
         throw new SyntaxError ("isequal: object types are not compatible");

      _testResult.log (name1 + " isequal " + name2 + " : " + status);
      }  // end do_isequal



   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *    the first (only?) of which is the timeout to be used
    *    which defaults to the value -1
    */
   protected void do_begin (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_begin");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      int timeout = -1;
      if (line.size () > 0)
         {
         String t = (String) line.remove (0);
         timeout = Integer.parseInt (t);
         }

      sm1.begin (timeout);

      }  // end do_begin


   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the completionstatus to be used
    */
   protected void do_completewithstatus (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_completewithstatus");
      if (line.size () == 0)
         throw new SyntaxError ("specify CompletionStatus");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String cs = (String) line.remove (0);
      int cstatus = 0;
      if (cs.equals ("success"))
         cstatus = CompletionStatus.CompletionStatusSuccess;
      else if (cs.equals ("fail"))
         cstatus = CompletionStatus.CompletionStatusFail;
      else if (cs.equals ("failonly"))
         cstatus = CompletionStatus.CompletionStatusFailOnly;
      else
         cstatus = Integer.parseInt (cs);
         // throw new SyntaxError ("Invalid completion status " + cs);

      Outcome outcome = sm1.completeWithStatus (cstatus);
      if (outcome != null)
         _testResult.log ("outcome from completewithstatus = " +
            outcome.getName ());
      else
         _testResult.log ("outcome from completewithstatus = null");

      }  // end do_completewithstatus


   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   returned <code>ActivityToken</code> will be stored in the
    *   global table
    */
   protected void do_suspend (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_suspend");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      ActivityToken tok = sm1.suspend ();
      if (tok != null)
         put (tokname, tok);
      else
         _testResult.log ("null token returned from suspend");

      }  // end do_suspend


   /**
    * Call the correspondng ActivityManager function
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   <code>ActivityToken</code> that is used in the resume
    *   has been stored in the global table
    */
   protected void do_resume (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_resume");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityToken)
         sm1.resume ((ActivityToken) o);
      else if (tokname.equals ("null"))
         sm1.resume ((ActivityToken) null);
      else
         throw new SyntaxError (tokname + " is invalid");

      }  // end do_resume


   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   returned <code>ActivityToken</code> will be stored in the
    *   global table
    */
   protected void do_suspendgroup (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_suspendgroup");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      ActivityToken tok = sm1.suspendGroup ();
      if (tok != null)
         put (tokname, tok);
      else
         _testResult.log ("null token returned from suspendGroup");

      }  // end do_suspendgroup


   /**
    * Call the correspondng ActivityManager function
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   <code>ActivityToken</code> that is used in the resumegroup
    *   has been stored in the global table
    */
   protected void do_resumegroup (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_resumegroup");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityToken)
         sm1.resumeGroup ((ActivityToken) o);
      else if (tokname.equals ("null"))
         sm1.resumeGroup ((ActivityToken) null);
      else
         throw new SyntaxError (tokname + " is invalid");

      }  // end do_resumegroup


   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   returned <code>ActivityToken</code> will be stored in the
    *   global table
    */
   protected void do_suspendall (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_suspendall");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      ActivityToken tok = sm1.suspendAll ();
      if (tok != null)
         put (tokname, tok);
      else
         _testResult.log ("null token returned from suspendAll");

      }  // end do_suspendall


   /**
    * Call the correspondng ActivityManager function
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   <code>ActivityToken</code> that is used in the resumeall
    *   has been stored in the global table
    */
   protected void do_resumeall (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_resumeall");
      if (line.size () == 0)
         throw new SyntaxError ("specify token name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityToken)
         sm1.resumeAll ((ActivityToken) o);
      else if (tokname.equals ("null"))
         sm1.resumeAll ((ActivityToken) null);
      else
         throw new SyntaxError (tokname + " is invalid");

      }  // end do_resumeall


   /**
    * Add a new <code>SignalSet</code> to the current ServiceManager.
    * @param  line contains the current command line parameters
    *    the first of which contains the name for the SignalSet.
    *    A second optional parameter indicates whether the SignalSet
    *    is to be used in <i>completion</i> processing.
    */
   protected void do_addsignalset (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_addsignalset");
      if (line.size () == 0)
         throw new SyntaxError ("specify signalsest name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      SignalSetImpl sigset = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof SignalSetImpl)
         {
         sigset = (SignalSetImpl) o;
         }
      else
         {
         _ref.println ("creating new signalset");
         sigset = new SignalSetImpl (tokname, _testResult);
         put (tokname, sigset);   // may overwrite existing item
         }

      // get completion data
      boolean css = false;
      if (line.size () > 0 &&
         ((String) line.remove (0)).equals ("true"))
         {
         _ref.println ("completion signalset");
         css = true;
         }

      sm1.addSignalSet (sigset, css);

      }  // end do_addsignalset


   /**
    * Create and optionally configure a <code>SignalSet</code>.
    * The SignalSet object is stored in the global table using the
    * name provided as the first command line parameter.
    * @param  line contains the current command line parameters.
    *    The command comes in several variations
    * <OL>
    * <LI> 'signalset {name} signal {signalname1 signalname2 ...}' adds a
    * series of <code>Signal</code>s to the SignalSet
    * <LI> 'signalset {name} response {respname actioninterested usenext}'
    * configures the SignalSet to respond to <code>Outcome</code>
    * respname with a <code>Signaling</code> object with the
    * corresponding actioninterested and usenext boolean values.
    * <LI> 'signalset {name} outcome {outcome}
    * </OL>
    */
   protected void do_signalset (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_signalset");
      // "signalset xxx response yyy true/false true/false"
      // "signalset xxx signal zzz"
      // "signalset xxx outcome aaa"
      if (line.size () == 0)
         throw new SyntaxError ("specify signalsest name");

      String sigsetname = (String) line.remove (0);
      if (line.size () == 0)
         throw new SyntaxError  ("specify action for signalset");

      SignalSetImpl sigset = null;
      Object o = get (sigsetname);
      if (o != null && o instanceof SignalSetImpl)
         {
         sigset = (SignalSetImpl) o;
         }
      else
         {
         _ref.println ("creating new signalset");
         sigset = new SignalSetImpl (sigsetname, _testResult);
         put (sigsetname, sigset);  //may overwrite existing item
         }

      String action = (String) line.remove (0);
      if (action.equals ("signal"))
         {
         while (line.size () > 0)
            sigset.addSignal ((String) line.remove (0));
         }
      else if (action.equals ("response"))
         {
         if (line.size () == 0)
            throw new SyntaxError  ("no response found for signalset");

         String resp = (String) line.remove (0);
         boolean b1 = true;
         boolean b2 = false;
         if (line.size () > 0 &&
            ((String) line.remove (0)).equals ("false"))
            b1 = false;
         if (line.size () > 0 &&
            ((String) line.remove (0)).equals ("true"))
            b2 = true;
         sigset.addResponse (resp, b1, b2);
         }
      else if (action.equals ("outcome"))
         {
         if (line.size () == 0)
            throw new SyntaxError  ("no outcome found for signalset");

         sigset.addOutcome ((String) line.remove (0));
         }
      else
         throw new SyntaxError ("invalid action for signalset");

      }  // end do_signalset


   /**
    * Add a new <code>PropertyGroupManager</code> to the current ServiceManager.
    * @param  line contains the current command line parameters
    *    the first of which contains the name for the PropertyGroupManager.
    */
   protected void do_addpropertygroupmanager (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_addpropertygroupmanager");
      if (line.size () == 0)
         throw new SyntaxError ("specify PropertyGroupManager name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      PropertyGroupManagerImpl pgm = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof PropertyGroupManagerImpl)
         {
         pgm = (PropertyGroupManagerImpl) o;
         }
      else
         {
         _ref.println ("creating new PropertyGroupManager");
         pgm = new PropertyGroupManagerImpl (tokname, _testResult);
         put (tokname, pgm);   // may overwrite existing item
         }

      sm1.addPropertyGroupManager (tokname, pgm);

      }  // end do_addpropertygroupmanager


   /**
    * Create and optionally configure a <code>PropertyGroupManager</code>.
    * The PropertyGroupManager object is stored in the global table using the
    * name provided as the first command line parameter.
    * @param  line contains the current command line parameters.
    *    The PropertyGroup that this PGM creates can be configured to
    *    execute a series of commands when the various interface methods
    *    are called by the Activity Service.
    */
   protected void do_propertygroupmanager (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_propertygroupmanager");
      // 'propertygroupmanager xxx'
      // 'propertygroupmanager xxx completed/suspended/resumed actions'
      if (line.size () == 0)
         throw new SyntaxError ("specify PropertyGroupManager name");

      String pgmname = (String) line.remove (0);

      PropertyGroupManagerImpl pgm = null;
      Object o = get (pgmname);
      if (o != null && o instanceof PropertyGroupManagerImpl)
         {
         pgm = (PropertyGroupManagerImpl) o;
         }
      else
         {
         _ref.println ("creating new PropertyGroupManager");
         pgm = new PropertyGroupManagerImpl (pgmname, _testResult);
         put (pgmname, pgm);  //may overwrite existing item
         }

      if (line.size () > 0)
         {
         String action = (String) line.remove (0);
         if (action.equals ("completed") ||
            action.equals ("suspended")  ||
            action.equals ("resumed") )
            {
            if (line.size () == 0)
               throw new SyntaxError (
                  "no commands found for PropertyGroupManager");

            String resp = (String) line.remove (0);
            pgm.addResponse (action, new TestCommand (resp, this));
            }
         else
            throw new SyntaxError ("invalid action for PropertyGroupManager");

         }  // end if

      }  // end do_propertygroupmanager


   /**
    * Create and optionally configure an <code>Action</code>.
    * The Action object is stored in the global table using the
    * name provided as the first command line parameter.
    * @param  line contains the current command line parameters.
    *    The command syntax has serveral options as follows
    * <OL>
    * <LI> 'action {name} {signal} {response}'
    * <br> which configures the Action to produce an <code>Outcome</code>
    * in response to the named <code>Signal</code>
    * <LI> 'action {name} {signal} actionerror
    * <br> which configures the Action to throw the ActionErrorException
    * in response to the Signal
    * <LI> 'action {name} {signal} actionsystemexception
    * <br> which configures the Action to throw a NullPointerException
    * in response to the Signal
    * <LI> 'action {name} {signal} command {cmdstring}'
    * <br> which configures the Action execute the test commands in
    * <cmdstrng> in response to the Signal
    * <LI> 'action {name} getglobalid {gidname}'
    * <br> which retrieves the  GlobalId that the Action extracted
    * from the extended data contained in the last Signal received.
    * <LI> 'action {name} getcmpletionstatus'
    * <br> which retrieves the CompletionStatus that the Action extracted
    * from the extended data contained in the last Signal received.
    * <LI> 'action {name} getfinaloutcome'
    * <br> which retrieves the  Outcome that the Action extracted
    * from the extended data contained in the last Signal received.
    * </OL>
    */
   protected void do_action (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_action");
      // action xxx signal outcome
      // action xxx signal command 'yyy yyy yyy'
      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      TestAction ta = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         {
         _ref.println ("creating new action");
         ta = new TestAction (tokname, _testResult);
         put (tokname, ta);   // may overwrite existing item
         }

      // now get signal name and outcome name
      String  signal = "";
      String  outcome = "";
      if (line.size () > 0)
          signal = (String) line.remove (0);

      if (line.size () > 0)
         outcome = (String) line.remove (0);

      // special case commands to extract Signal extended data
      if (signal.equals ("getglobalid"))
         {
         GlobalId gid = ta.getGlobalId ();
         if (gid == null)
            _testResult.log ("action.getGlobalId returned null");
         else
            put (outcome, gid);
         }
      else if (signal.equals ("getfinaloutcome"))
         {
         Outcome oc = ta.getFinalOutcome ();
         if (oc == null)
            _testResult.log (
               "action(" + tokname + ").getFinalOutcome returned null");
         else
            _testResult.log (
               "action(" + tokname + ").getFinalOutcome returned " +
               oc.getName ());
         }
      else if (signal.equals ("getcompletionstatus"))
         {
         int compstatus = ta.getCompletionStatus ();
         String cs = null;
         switch (compstatus)
            {
         case CompletionStatus.CompletionStatusSuccess:
            cs = "CompletionStatusSuccess";
            break;
         case CompletionStatus.CompletionStatusFail:
            cs = "CompletionStatusFail";
            break;
         case CompletionStatus.CompletionStatusFailOnly:
            cs = "CompletionStatusFailOnly";
            break;
         default :
            cs = "" + compstatus;
            }

         _testResult.log ("action(" + tokname +
            ").getCompletionStatus returns " + cs);
         }
      else if (outcome.equals ("command"))
         {
         String cmds = null;
         if (line.size () > 0)
            cmds = (String) line.remove (0);

         ta.addCommand (signal, new TestCommand (cmds, this));
         }  // end if
      else
         ta.addResponse (signal, outcome);

      }  // end do_action



   /**
    * Call the correspondng ActivityManager function.
    * Adds an <code>Action</code> to the current Activity.
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'addaction actionname signalsetname priority'
    * <br>
    */
   protected void do_addaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_addaction");
      // addaction xxx sigset priority
      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      TestAction ta = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         {
         _ref.println ("creating new action");
         ta = new TestAction (tokname, _testResult);
         put (tokname, ta);  // may overwrite existing item
         }

      // get sigsetname
      if (line.size () == 0)
         throw new SyntaxError ("specify signalset name");

      String sigset   = (String) line.remove (0);
      int    priority = 0;
      if (line.size () > 0)
         priority = Integer.parseInt ((String) line.remove (0));

      sm1.addAction (ta, sigset, priority);

      }  // end do_addaction



   /**
    * Call the correspondng ActivityManager function.
    * Adds a Global <code>Action</code> to the current Activity.
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'addglobalaction actionname priority'
    * <br>
    */
   protected void do_addglobalaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_addglobalaction");
      // addglobalaction xxx priority
      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      TestAction ta = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         {
         _ref.println ("creating new action");
         ta = new TestAction (tokname, _testResult);
         put (tokname, ta);  // may overwrite existing item
         }

      int    priority = 0;
      if (line.size () > 0)
         priority = Integer.parseInt ((String) line.remove (0));

      sm1.addGlobalAction (ta, priority);

      }  // end do_addglobalaction



   /**
    * Call the correspondng ActivityManager function.
    * Removes an <code>Action</code> from current Activity.
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'removeaction actionname'
    * <br>
    */
   protected void do_removeaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_removeaction");
      // removeaction xxx
      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      TestAction ta = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         throw new SyntaxError (tokname + " is not a valid Action");

      if (line.size () == 0)
         throw new SyntaxError ("specify signalset name");

      String ssname = (String) line.remove (0);

      sm1.removeAction (ta, ssname);

      }  // end do_removeaction



   /**
    * Call the correspondng ActivityManager function.
    * Removes a Global <code>Action</code> from current Activity.
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'removeglobalaction actionname'
    * <br>
    */
   protected void do_removeglobalaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_removeglobalaction");
      // removeglobalaction xxx
      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      TestAction ta = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         throw new SyntaxError (tokname + " is not a valid Action");

      sm1.removeGlobalAction (ta);

      }  // end do_removeglobalaction



   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   returned <code>GlobalId</code> will be stored in the
    *   global table
    */
   protected void do_getglobalid (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getglobalid");
      if (line.size () == 0)
         throw new SyntaxError ("specify GlobalId name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      GlobalId tok = sm1.getGlobalId ();
      if (tok != null)
         put (tokname, tok);
      else
         _testResult.log ("null GlobalId returned from getGlobalId");

      }  // end do_getglobalid


   /**
    * Call the correspondng ActivityManager function.
    * @param  line contains the current command line parameters
    *   the first (only?) of which is the name/key under which the
    *   returned <i>name</i> String will be stored in the
    *   global table
    */
   protected void do_getname (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getname");
      if (line.size () == 0)
         throw new SyntaxError ("specify name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String tokname = (String) line.remove (0);
      String tok = sm1.getName ();
      if (tok != null)
         put (tokname, tok);
      else
         _testResult.log ("null String returned from getName");

      }  // end do_getname


   /**
    * Call the correspondng ActivityManager function.
    * Sets the <code>CompletionStatus</code> of the current Activity.
    * @param  line contains the current command line parameters.
    *     the only parameter for this command is the CompletionStatus
    *     to set.  The text values 'success', 'fail' and 'failonly'
    *     are recognized and converted to the corresponding
    *     CompletionStatus integers.  Any other value is treated as
    *     a number and a NumberFormatException will be thrown if it is not.
    */
   protected void do_setcompletionstatus (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_setcompletionstatus");
      // setcompletionstatus xxx
      if (line.size () == 0)
         throw new SyntaxError ("specify completion status");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String cs = (String) line.remove (0);
      int cstatus = 0;
      if (cs.equals ("success"))
         cstatus = CompletionStatus.CompletionStatusSuccess;
      else if (cs.equals ("fail"))
         cstatus = CompletionStatus.CompletionStatusFail;
      else if (cs.equals ("failonly"))
         cstatus = CompletionStatus.CompletionStatusFailOnly;
      else
         cstatus = Integer.parseInt (cs);
         // throw new SyntaxError ("Invalid completion status " + cs);

      sm1.setCompletionStatus (cstatus);

      }  // end do_setcompletionstatus



   /**
    * Call the correspondng ActivityManager function.
    * Sets the timeout of the current ActivityManager
    * @param  line contains the current command line parameters.
    *     the only parameter for this command is the timeout
    */
   protected void do_settimeout (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_settimeout");
      // settimeout xxx

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      int timeout = 0;

      if (line.size () > 0)
         {
         String cs = (String) line.remove (0);
         timeout = Integer.parseInt (cs);
         }

      sm1.setTimeout (timeout);

      }  // end do_settimeout



   /**
    * Call the correspondng ActivityManager function.
    * calls the 'broadCast' method with the name of the signalset or null
    * @param  line contains the current command line parameters.
    *     the only parameter for this command is the signalset name
    */
   protected void do_broadcast (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_broadcast");
      // broadcast xxx

      if (sm1 == null)
         throw new SyntaxError ("No service manager");


      String cs = null;
      if (line.size () > 0)
         {
         cs = (String) line.remove (0);
         }

      Outcome outcome = sm1.broadcast (cs);
      if (outcome != null)
         _testResult.log ("outcome from broadcast = " +
            outcome.getName ());
      else
         _testResult.log ("outcome from broadcast = null");

      }  // end do_broadcast



   /**
    * Call the correspondng ActivityManager function.
    * calls the 'getPropertyGroup' method with the name of the propertyGroup
    * @param  line contains the current command line parameters.
    *     the only parameter for this command is the PG name
    */
   protected void do_getpropertygroup (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getpropertygroup");
      // getpropertygroup xxx

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      String pgm = null;
      if (line.size () > 0)
         {
         pgm = (String) line.remove (0);
         }

      PropertyGroup pg = sm1.getPropertyGroup (pgm);
      if (pg != null)
         _testResult.log ("outcome from getpropertygroup = " +
            pg.getPropertyGroupName ());
      else
         _testResult.log ("outcome from getpropertygroup = null");

      }  // end do_getpropertygroup



   /**
    * Call the correspondng ActivityCoordinator function.
    *
    * @param  line contains the current command line parameters.
    *     there are two parameters, the name of the ActivityCoordinator
    *     to use and the CompletionStatus.  As with other functions,
    *     the Strings 'success', 'fail' and 'failonly' are recognized.
    */
   protected void do_completeactivity (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_completeactivity");
      // completeactivity <name> <completionstatus>
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");
         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify completion status");

      String cs = (String) line.remove (0);
      int cstatus = 0;
      if (cs.equals ("success"))
         cstatus = CompletionStatus.CompletionStatusSuccess;
      else if (cs.equals ("fail"))
         cstatus = CompletionStatus.CompletionStatusFail;
      else if (cs.equals ("failonly"))
         cstatus = CompletionStatus.CompletionStatusFailOnly;
      else
         cstatus = Integer.parseInt (cs);
         // throw new SyntaxError ("Invalid completion status " + cs);

      Trace.out (">>> ActivityCoordinator.completeActivity");
      Outcome outcome = ac.completeActivity (cstatus);
      Trace.out ("<<< ActivityCoordinator.completeActivity");
      if (outcome == null)
         _testResult.log ("completeActivity returned null");
      else
         _testResult.log ("completeActivity returned " + outcome.getName ());

      }  // end do_completeactivity



   /**
    * Call the correspondng ActivityCoordinator function.
    *
    * @param  line contains the current command line parameters.
    *     there are two parameters, the name of the ActivityCoordinator
    *     to use and the CompletionStatus.  As with other functions,
    *     the Strings 'success', 'fail' and 'failonly' are recognized.
    */
   protected void do_heuristiccomplete (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_heuristiccomplete");
      // heuristiccomplete <name> <completionstatus>
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");
         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify completion status");

      String cs = (String) line.remove (0);
      int cstatus = 0;
      if (cs.equals ("success"))
         cstatus = CompletionStatus.CompletionStatusSuccess;
      else if (cs.equals ("fail"))
         cstatus = CompletionStatus.CompletionStatusFail;
      else if (cs.equals ("failonly"))
         cstatus = CompletionStatus.CompletionStatusFailOnly;
      else
         cstatus = Integer.parseInt (cs);
         // throw new SyntaxError ("Invalid completion status " + cs);

      Trace.out (">>> ActivityCoordinator.heuristicComplete");
      Outcome outcome = ac.heuristicComplete (cstatus);
      Trace.out ("<<< ActivityCoordinator.heuristicComplete");
      if (outcome == null)
         _testResult.log ("heuristicComplete returned null");
      else
         _testResult.log ("heuristicComplete returned " + outcome.getName ());

      }  // end do_heuristiccomplete



   /**
    * Call the correspondng ActivityCoordinator function.
    *
    * @param  line contains the current command line parameters.
    *     there are three parameters, the name of the ActivityCoordinator
    *     to use, the SignalSet name and the completionstatus
    */
   protected void do_processsignalset (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_processsignalset");
      // processsignalset <name> <signalsetname> <completionstatus>
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");
         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify SignalSet name");

      String sigset = (String) line.remove (0);

      if (line.size () == 0)
         throw new SyntaxError ("specify completion status");

      String cs = (String) line.remove (0);
      int cstatus = 0;
      if (cs.equals ("success"))
         cstatus = CompletionStatus.CompletionStatusSuccess;
      else if (cs.equals ("fail"))
         cstatus = CompletionStatus.CompletionStatusFail;
      else if (cs.equals ("failonly"))
         cstatus = CompletionStatus.CompletionStatusFailOnly;
      else
         cstatus = Integer.parseInt (cs);
         // throw new SyntaxError ("Invalid completion status " + cs);

      Trace.out (">>> ActivityCoordinator.processSignalSet");
      Outcome outcome = ac.processSignalSet (sigset, cstatus);
      Trace.out ("<<< ActivityCoordinator.processSignalSet");
      if (outcome == null)
         _testResult.log ("processSignalSet returned null");
      else
         _testResult.log ("processSignalSet returned " + outcome.getName ());

      }  // end do_processsignalset



   /**
    * Extract and store a reference to the current Activity's
    * ActivityCoordinator
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'activitycoordinator name'
    * <br>
    */
   protected void do_activitycoordinator (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_activitycoordinator");
      // activitycoordinator name
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         if (ac != null)
            put (tokname, ac);  // may overwrite existing item
         else
            _testResult.log ("Command 'activitycoordinator' returned null");
         }

      }  // end do_activitycoordinator



   /**
    * Extract and store a reference to the current Activity's
    * parent ActivityCoordinator
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getparentcoordinator name'
    * <br>
    */
   protected void do_getparentcoordinator (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getparentcoordinator");
      // getparentcoordinator name
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      if (sm1 == null)
         throw new SyntaxError ("No service manager");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getParentCoordinator ();
         if (ac != null)
            put (tokname, ac);  // may overwrite existing item
         else
            _testResult.log ("Command 'getparentcoordinator' returned null");
         }

      }  // end do_getparentcoordinator



   /**
    * Call the ActivityCoordinator addAction method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordaddaction coordname actname sigsetname priority'
    * <br>
    */
   protected void do_coordaddaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordaddaction");
      // coordaddaction coordname actname sigsetname priority
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      TestAction ta = null;
      tokname = (String) line.remove (0);
      o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         {
         _ref.println ("creating new action");
         ta = new TestAction (tokname, _testResult);
         put (tokname, ta);  // may overwrite existing item
         }

      // get sigsetname
      if (line.size () == 0)
         throw new SyntaxError ("specify signalset name");

      String sigset   = (String) line.remove (0);
      int    priority = 0;
      if (line.size () > 0)
         priority = Integer.parseInt ((String) line.remove (0));

      Trace.out (">>> ActivityCoordinator.addAction");
      ac.addAction (ta, sigset, priority);
      Trace.out ("<<< ActivityCoordinator.addAction");

      }  // end do_coordaddaction



   /**
    * Call the ActivityCoordinator addGlobalAction method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordaddglobalaction coordname actname priority'
    * <br>
    */
   protected void do_coordaddglobalaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordaddglobalaction");
      // coordaddglobalaction coordname actname priority
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify action name");

      TestAction ta = null;
      tokname = (String) line.remove (0);
      o = get (tokname);
      if (o != null && o instanceof TestAction)
         {
         ta = (TestAction) o;
         }
      else
         {
         _ref.println ("creating new action");
         ta = new TestAction (tokname, _testResult);
         put (tokname, ta);  // may overwrite existing item
         }

      int    priority = 0;
      if (line.size () > 0)
         priority = Integer.parseInt ((String) line.remove (0));

      Trace.out (">>> ActivityCoordinator.addGlobalAction");
      ac.addGlobalAction (ta, priority);
      Trace.out ("<<< ActivityCoordinator.addGlobalAction");

      }  // end do_coordaddglobalaction



   /**
    * Call the ActivityCoordinator getNumberRegisteredActions method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getactioncount coordname signalset'
    * <br>
    */
   protected void do_getactioncount (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getactioncount");
      // getactioncount coordname signalset
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify SignalSet name");

      String sigset = (String) line.remove (0);

      Trace.out (">>> ActivityCoordinator.getNumberRegisteredActions");
      int acount = ac.getNumberRegisteredActions (sigset);
      Trace.out ("<<< ActivityCoordinator.getNumberRegisteredActions");
      _testResult.log ("getNumberRegisteredActions returned " + acount);

      }  // end do_getactioncount



   /**
    * Call the ActivityCoordinator setCompletionSignalSetName method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'setcompletionsignalsetname coordname signalset'
    * <br>
    */
   protected void do_setcssname (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_setcssname");
      // setcompletionsignalsetname coordname signalset
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify SignalSet name");

      String sigset = (String) line.remove (0);
      if (sigset.equals("null"))
         sigset = null;

      Trace.out (">>> ActivityCoordinator.setCompletionSignalSetName");
      ac.setCompletionSignalSetName (sigset);
      Trace.out ("<<< ActivityCoordinator.setCompletionSignalSetName");

      }  // end do_setcssname



   /**
    * Call the ActivityCoordinator getCompletionSignalSetName method
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getcompletionsignalsetname coordname'
    * <br>
    */
   protected void do_getcssname (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getcssname");
      // getcompletionsignalsetname coordname s
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }


      String sigset = null;

      Trace.out (">>> ActivityCoordinator.getCompletionSignalSetName");
      sigset = ac.getCompletionSignalSetName ();
      Trace.out ("<<< ActivityCoordinator.getCompletionSignalSetName");
      _testResult.log ("getCompletionSignalSetName returned " + sigset);

      }  // end do_getcssname



   /**
    * Call the ActivityCoordinator getActions method
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getactions coordname'
    * <br>
    */
   protected void do_getactions (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getactions");
      // getactions coordname
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      String sigset = null;
      if (line.size () > 0)        // no name == null
         {
         sigset = (String) line.remove (0);
         if (sigset.equals("null")) sigset = null;
         }

      Trace.out (">>> ActivityCoordinator.getActions");
      Action[] alist = ac.getActions (sigset);
      Trace.out ("<<< ActivityCoordinator.getActions");
      _testResult.log ("getActions returned " + printActions (alist));

      }  // end do_getactions


   /**
    * Private method to turn array of actions into sorted unique
    * string of Action names
    */
   private String printActions (Action[] alist)
      {
      String result = new String();
      if (alist == null)
         return "<null>";

      Arrays.sort (alist);
      for (int i = 0; i < alist.length; i++)
         result = result + alist[i] + " ";

      return result;
      }


   /**
    * Call the ActivityCoordinator getParent method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getparent coordname parentname'
    * <br>
    */
   protected void do_getparent (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getparent");
      // getparent coordname parentname
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify parent Coordinator name");

      String parentname = (String) line.remove (0);
      Trace.out (">>> ActivityCoordinator.getParent");
      ActivityCoordinator pc = ac.getParent ();
      Trace.out ("<<< ActivityCoordinator.getParent");
      if (pc == null)
         _testResult.log ("getParent returned null");
      else
         put (parentname, pc);
      }  // end do_getparent



   /**
    * Call the ActivityCoordinator getGlobalId method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordgetglobalid coordname globalid'
    * <br>
    */
   protected void do_coordgetglobalid (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordgetglobalid");
      // coordgetglobalid coordname globalid
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify GlobalId name");

      String gidname = (String) line.remove (0);
      Trace.out (">>> ActivityCoordinator.getGlobalId");
      GlobalId gid = ac.getGlobalId ();
      Trace.out ("<<< ActivityCoordinator.getGlobalId");
      if (gid == null)
         _testResult.log ("getGlobalId returned null");
      else
         put (gidname, gid);
      }  // end do_coordgetglobalid



   /**
    * Call the ActivityCoordinator getName method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordgetname coordname name'
    * <br>
    */
   protected void do_coordgetname (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordgetname");
      // coordgetname coordname name
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify name");

      String namename = (String) line.remove (0);
      Trace.out (">>> ActivityCoordinator.getName");
      String name = ac.getName ();
      Trace.out ("<<< ActivityCoordinator.getName");
      if (name == null)
         _testResult.log ("getName returned null");
      else
         put (namename, name);
      }  // end do_coordgetname


      /**
       * Call the ActivityCoordinator.setPersistent method.
       * @param  line contains the current command line parameters.
       *    The syntax of the command is as follows
       * <br> 'setpersistent coordname <id>'
       * <br> where id is optional for saving returned RecreateData
       */
      protected void do_coordsetpersistent (Vector line) throws Exception
         {
         Trace.out ("TestActivity.do_coordsetpersistent");
         // setpersistent coordname
         if (line.size () == 0)
            throw new SyntaxError ("specify ActivityCoordinator name");

         ActivityCoordinator ac = null;
         String tokname = (String) line.remove (0);
         Object o = get (tokname);
         if (o != null && o instanceof ActivityCoordinator)
            {
            ac = (ActivityCoordinator) o;
            }
         else
            {
            throw new SyntaxError ("No Activity to test");
            }

         Trace.out (">>> ActivityCoordinator.setPersistent");
         RecreateData result = ac.setPersistent();
         Trace.out ("<<< ActivityCoordinator.setPersistent");
         if (line.size () > 0)
            {
            String namename = (String) line.remove (0);
            put (namename, result);
            }

         }  // end do_coordsetpersistent


      /**
       * Call the ActivityCoordinator hibernate method directly
       * @param  line contains the current command line parameters.
       *    The syntax of the command is as follows
       * <br> 'coordhibernate coordname <id>'
       * <br> where id is optional for saving returned RecreateData
       */
      protected void do_coordhibernate (Vector line) throws Exception
         {
         Trace.out ("TestActivity.do_coordhibernate");
         // coordhibernate coordname
         if (line.size () == 0)
            throw new SyntaxError ("specify ActivityCoordinator name");

         ActivityCoordinator ac = null;
         String tokname = (String) line.remove (0);
         Object o = get (tokname);
         if (o != null && o instanceof ActivityCoordinator)
            {
            ac = (ActivityCoordinator) o;
            }
         else
            {
            throw new SyntaxError ("No Activity to Hibernate");
            }

         Trace.out (">>> ActivityCoordinator.hibernate");
         RecreateData result = ac.hibernate();
         Trace.out ("<<< ActivityCoordinator.hibernate");

         if (result == null)
            _testResult.log ("hibernate returned null");

         if (line.size () > 0)
            {
            String namename = (String) line.remove (0);
            put (namename, result);
            }

         }


      /**
       * Call the ActivityCoordinator completeactivity method directly
       * @param  line contains the current command line parameters.
       *    The syntax of the command is as follows
       * <br> 'completeactivity coordname status'
       * <br>
       */
      protected void do_coordcompleteactivity (Vector line) throws Exception
         {
         Trace.out ("TestActivity.do_coordcompleteactivity");
         // completeactivity coordname status
         if (line.size () == 0)
            throw new SyntaxError ("specify ActivityCoordinator name");

         ActivityCoordinator ac = null;
         String tokname = (String) line.remove (0);
         Object o = get (tokname);
         if (o != null && o instanceof ActivityCoordinator)
            {
            ac = (ActivityCoordinator) o;
            }
         else
            {
            throw new SyntaxError ("No Activity to complete");
            }

         if (line.size () == 0)
            throw new SyntaxError ("specify completion status");

         String cs = (String) line.remove (0);
         int cstatus = 0;
         if (cs.equals ("success"))
            cstatus = CompletionStatus.CompletionStatusSuccess;
         else if (cs.equals ("fail"))
            cstatus = CompletionStatus.CompletionStatusFail;
         else if (cs.equals ("failonly"))
            cstatus = CompletionStatus.CompletionStatusFailOnly;
         else
            cstatus = Integer.parseInt (cs);
           // throw new SyntaxError ("Invalid completion status " + cs);


         Trace.out (">>> ActivityCoordinator.completeActivity");
         Outcome result = ac.completeActivity (cstatus);
         Trace.out ("<<< ActivityCoordinator.completeActivity");

         if (result == null)
            _testResult.log ("completeActivity returned null");
         else
            _testResult.log ("completeActivity returned " + result.getName());
         }


      /**
       * Call the UserActivity recreate method directly
       * @param  line contains the current command line parameters.
       *    The syntax of the command is as follows
       * <br> 'recreate activity parent resume recdata name'
       * <br>
       */
      protected void do_recreate (Vector line) throws Exception
         {
         Trace.out ("TestActivity.do_recreate");

         if (line.size () == 0)
            throw new SyntaxError ("specify activity GlobalId");

         String gidname = (String) line.remove (0);
         GlobalId activity = null;
         if (!"null".equals(gidname)) activity = (GlobalId) get (gidname);

         if (line.size () == 0)
            throw new SyntaxError ("specify parent GlobalId");

         String parentgidname = (String) line.remove (0);
         GlobalId parent = null;
         if (!"null".equals(parentgidname))
            parent = (GlobalId) get (parentgidname);

         if (line.size () == 0)
            throw new SyntaxError ("specify resume flag");
         String resumeflagstring = (String) line.remove (0);
         boolean resumeflag = "true".equals (resumeflagstring);

         RecreateData recData = null;
         if (line.size () == 0)
            throw new SyntaxError ("specify RecreateData value");
         String recDataToken = (String) line.remove (0);
         if (!recDataToken.equals("null"))
            {
            Object o = get (recDataToken);
            if (o == null || o instanceof RecreateData)
               {
               recData = (RecreateData) o;
               }
            else
               {
               throw new SyntaxError ("Invalid RecreateData");
               }

            }


         if (line.size () == 0)
            throw new SyntaxError ("specify name");
         String name = (String) line.remove (0);

         Trace.out (">>> UserActivity.recreate");
         ActivityCoordinator result = sm1.recreate (activity, parent, resumeflag, recData);
         Trace.out ("<<< UserActivity.recreate");

         if (result == null)
            _testResult.log ("recreate returned null");
         else
            put (name, result);
         }

   /**
    * Call the ActivityCoordinator getStatus method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordgetstatus coordname'
    * <br>
    */
   protected void do_coordgetstatus (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordgetstatus");
      // coordgetstatus coordname
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      Trace.out (">>> ActivityCoordinator.getStatus");
      int status = ac.getStatus ();
      Trace.out ("<<< ActivityCoordinator.getStatus");
      String  cs = null;
      switch (status)
         {
      case Status.StatusActive:
         cs = "StatusActive";
         break;
      case Status.StatusCompleted:
         cs = "StatusCompleted";
         break;
      case Status.StatusCompleting:
         cs = "StatusCompleting";
         break;
      case Status.StatusCompletingHeuristic:
         cs = "StatusCompletingHeuristic";
         break;
      case Status.StatusNoActivity:
         cs = "StatusNoActivity";
         break;
      // case Status.StatusUnknown:
      //    cs = "StatusUnknown";
      //    break;
      default :
         cs = "" + status;
         }
      _testResult.log ("ActivityCoordinator getStatus returned " + cs);
      }  // end do_coordgetstatus



   /**
    * Call the ActivityCoordinator getParentStatus method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'getparentstatus coordname'
    * <br>
    */
   protected void do_getparentstatus (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_getparentstatus");
      // getparentstatus coordname
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      Trace.out (">>> ActivityCoordinator.getParentStatus");
      int status = ac.getParentStatus ();
      Trace.out ("<<< ActivityCoordinator.getParentStatus");
      String  cs = null;
      switch (status)
         {
      case Status.StatusActive:
         cs = "StatusActive";
         break;
      case Status.StatusCompleted:
         cs = "StatusCompleted";
         break;
      case Status.StatusCompleting:
         cs = "StatusCompleting";
         break;
      case Status.StatusCompletingHeuristic:
         cs = "StatusCompletingHeuristic";
         break;
      case Status.StatusNoActivity:
         cs = "StatusNoActivity";
         break;
      // case Status.StatusUnknown:
      //    cs = "StatusUnknown";
      //    break;
      default :
         cs = "" + status;
         }
      _testResult.log ("ActivityCoordinator getParentStatus returned " + cs);
      }  // end do_getparentstatus



   /**
    * Call the ActivityCoordinator isSameActivity method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'issameactivity coordname1 coordname2 '
    * <br>
    */
   protected void do_issameactivity (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_issameactivity");
      // issameactivity coordname1 coordname2
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify second ActivityCoordinator name");

      ActivityCoordinator oc = null;
      tokname = (String) line.remove (0);
      o = get (tokname);
      if (o != null  && o instanceof ActivityCoordinator)
         {
         oc = (ActivityCoordinator) o;
         }
      else
         {
         throw new SyntaxError (tokname + " is not valid");
         }

      Trace.out (">>> ActivityCoordinator.isSameActivity");
      boolean status = ac.isSameActivity (oc);
      Trace.out ("<<< ActivityCoordinator.isSameActivity");
      _testResult.log ("ActivityCoordinator isSameActivity returned " + status);
      }  // end do_issameactivity



   /**
    * Call the ActivityCoordinator removeAction method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordremoveaction coordname1 action'
    * <br>
    */
   protected void do_coordremoveaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordremoveaction");
      // coordremoveaction coord action
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify Action name");

      Action oc = null;
      tokname = (String) line.remove (0);
      o = get (tokname);
      if (o != null  && o instanceof TestAction)
         {
         oc = (Action) o;
         }
      else
         {
         throw new SyntaxError (tokname + " is not valid");
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify signalset name");

      String ssname = (String) line.remove (0);

      Trace.out (">>> ActivityCoordinator.removeAction");
      ac.removeAction (oc, ssname);
      Trace.out ("<<< ActivityCoordinator.removeAction");
      }  // end do_coordremoveaction



   /**
    * Call the ActivityCoordinator removeGlobalAction method directly
    * @param  line contains the current command line parameters.
    *    The syntax of the command is as follows
    * <br> 'coordremoveglobalaction coordname1 action'
    * <br>
    */
   protected void do_coordremoveglobalaction (Vector line) throws Exception
      {
      Trace.out ("TestActivity.do_coordremoveglobalaction");
      // coordremoveglobalaction coord action
      if (line.size () == 0)
         throw new SyntaxError ("specify ActivityCoordinator name");

      ActivityCoordinator ac = null;
      String tokname = (String) line.remove (0);
      Object o = get (tokname);
      if (o != null && o instanceof ActivityCoordinator)
         {
         ac = (ActivityCoordinator) o;
         }
      else
         {
         if (sm1 == null)
            throw new SyntaxError ("No service manager");

         _ref.println ("creating new ActivityCoordinator");
         ac = sm1.getCoordinator ();
         put (tokname, ac);  // may overwrite existing item
         }

      if (line.size () == 0)
         throw new SyntaxError ("specify Action name");

      Action oc = null;
      tokname = (String) line.remove (0);
      o = get (tokname);
      if (o != null  && o instanceof TestAction)
         {
         oc = (Action) o;
         }
      else
         {
         throw new SyntaxError (tokname + " is not valid");
         }

      Trace.out (">>> ActivityCoordinator.removeGlobalAction");
      ac.removeGlobalAction (oc);
      Trace.out ("<<< ActivityCoordinator.removeGlobalAction");
      }  // end do_coordremoveglobalaction



   /**
    * Implementation of the Thread interface method.
    * This is the central command processing loop which reads a
    * line from the inputstream and interprets the corresponding command.
    * The threads are arranged so that only one at a time is reading and
    * executing commands (the others are blocked on a semaphore).
    */
   public void run ()
      {
      Trace.out ("TestActivity.run");
      Vector line = null;
      while (!finished)
         {
         _ref.println ("thread " +
            Thread.currentThread() + " about to sleep");
         acquire ();
         _ref.println ("thread " +
            Thread.currentThread() + " activated");

         while (!finished && ((line = readLine ()) != null) )
            {
            if (executeCmd (line))
               break;
            }  // end while

         if (line == null)
            terminate ();    // finished = true;

         }  // end while

      }  // end run

   /**
    * Execute a single command line.
    * This is part of the central command processing loop which reads a
    * line from the inputstream and interprets the corresponding command.
    * Most valid commands result in the corresponding do_xxx method
    * being called.
    * @param line   is a Vector of Strings representing the 'tokens'
    *    extracted from the command line.
    * @return  true to put this thread to sleep.
    */
   public boolean executeCmd (Vector line)
      {
      // now parse line and do stuff
      int i = 0;
      for (i = 0; i < line.size (); i++)
         _ref.print (line.elementAt (i) + " - ");

      if (i > 0)
         _ref.println ("");

      if (line.size () == 0)  // nothing to do
         return false;

      String command = (String) line.remove (0);
      // enclose all commands in try/catch so diagnostics can be taken
      try {
         if (command.equals ("end"))
            {
            terminate ();
            return true;
            }
         else if (command.equals ("thread"))
            {
            if (line.size () > 0)
               {
               String threadName = (String) line.remove (0);
               TestActivity thrd = (TestActivity) selectThread (threadName);
               if (thrd != null)
                  {
                  thrd.release ();     // wake up selected thread
                  return true;        // and go to sleep
                  }
               }
            }
         else if (command.equals ("dump"))
            {
            _testResult.dump (_log);
            }
         else if (command.equals ("servicemanager"))
            {
            selectServiceMgr (line);
            }
         else if (command.equals ("begin"))
            {
            do_begin (line);
            }
         else if (command.equals ("complete"))
            {
            if (sm1 != null)
               {
               Outcome outcome = sm1.complete ();
               if (outcome != null)
                  _testResult.log ("outcome from complete = " +
                     outcome.getName ());
               else
                  _testResult.log ("outcome from complete = null");
               }
            else
               throw new SyntaxError ("No service manager");
            }
         else if (command.equals ("completewithstatus"))
            {
            do_completewithstatus (line);
            }
         else if (command.equals ("broadcast"))
            {
            do_broadcast (line);
            }
         else if (command.equals ("suspend"))
            {
            do_suspend (line);
            }
         else if (command.equals ("resume"))
            {
            do_resume (line);
            }
         else if (command.equals ("suspendgroup"))
            {
            do_suspendgroup (line);
            }
         else if (command.equals ("resumegroup"))
            {
            do_resumegroup (line);
            }
         else if (command.equals ("suspendall"))
            {
            do_suspendall (line);
            }
         else if (command.equals ("resumeall"))
            {
            do_resumeall (line);
            }
         else if (command.equals ("addsignalset"))
            {
            do_addsignalset (line);
            }
         else if (command.equals ("signalset"))
            {
            do_signalset (line);
            }
         else if (command.equals ("addpropertygroupmanager"))
            {
            do_addpropertygroupmanager (line);
            }
         else if (command.equals ("propertygroupmanager"))
            {
            do_propertygroupmanager (line);
            }
         else if (command.equals ("getname"))
            {
            do_getname (line);
            }
         else if (command.equals ("getglobalid"))
            {
            do_getglobalid (line);
            }
         else if (command.equals ("getservice"))
            {
            if (sm1 != null)
               {
               sm1.getService ();
               }
            else
               throw new SyntaxError ("No service manager");
            }
         else if (command.equals ("setcompletionstatus"))
            {
            do_setcompletionstatus (line);
            }
         else if (command.equals ("getcompletionstatus"))
            {
            if (sm1 != null)
               {
               int cs = sm1.getCompletionStatus ();
               }
            else
               throw new SyntaxError ("No service manager");
            }
         else if (command.equals ("settimeout"))
            {
            do_settimeout (line);
            }
         else if (command.equals ("gettimeout"))
            {
            if (sm1 != null)
               {
               int cs = sm1.getTimeout ();
               _testResult.log ("getTimeout returned " + cs);
               }
            else
               throw new SyntaxError ("No service manager");
            }
         else if (command.equals ("getpropertygroup"))
            {
            do_getpropertygroup (line);
            }
         else if (command.equals ("getstatus"))
            {
            if (sm1 != null)
               {
               int cs = sm1.getStatus ();
               }
            else
               throw new SyntaxError ("No service manager");
            }
         else if (command.equals ("action"))
            {
            do_action (line);
            }
         else if (command.equals ("addaction"))
            {
            do_addaction (line);
            }
         else if (command.equals ("coordaddaction"))
            {
            do_coordaddaction (line);
            }
         else if (command.equals ("coordaddglobalaction"))
            {
            do_coordaddglobalaction (line);
            }
         else if (command.equals ("addglobalaction"))
            {
            do_addglobalaction (line);
            }
         else if (command.equals ("removeaction"))
            {
            do_removeaction (line);
            }
         else if (command.equals ("removeglobalaction"))
            {
            do_removeglobalaction (line);
            }
         else if (command.equals ("activitycoordinator"))
            {
            do_activitycoordinator (line);
            }
         else if (command.equals ("getparentcoordinator"))
            {
            do_getparentcoordinator (line);
            }
         else if (command.equals ("processsignalset"))
            {
            do_processsignalset (line);
            }
         else if (command.equals ("completeactivity"))
            {
            do_completeactivity (line);
            }
         else if (command.equals ("heuristiccomplete"))
            {
            do_heuristiccomplete (line);
            }
         else if (command.equals ("getactioncount"))
            {
            do_getactioncount (line);
            }
         else if (command.equals ("getactions"))
            {
            do_getactions (line);
            }
         else if (command.equals ("getcompletionsignalsetname"))
            {
            do_getcssname (line);
            }
         else if (command.equals ("setcompletionsignalsetname"))
            {
            do_setcssname (line);
            }
         else if (command.equals ("coordgetstatus"))
            {
            do_coordgetstatus (line);
            }
         else if (command.equals ("getparentstatus"))
            {
            do_getparentstatus (line);
            }
         else if (command.equals ("getparent"))
            {
            do_getparent (line);
            }
         else if (command.equals ("coordgetglobalid"))
            {
            do_coordgetglobalid (line);
            }
         else if (command.equals ("coordgetname"))
            {
            do_coordgetname (line);
            }
         else if (command.equals ("issameactivity"))
            {
            do_issameactivity (line);
            }
         else if (command.equals ("coordremoveaction"))
            {
            do_coordremoveaction (line);
            }
         else if (command.equals ("coordhibernate"))
            {
            do_coordhibernate (line);
            }
         else if (command.equals ("completeactivity"))
            {
            do_coordcompleteactivity(line);
            }
         else if (command.equals ("setpersistent"))
            {
            do_coordsetpersistent(line);
            }
         else if (command.equals ("recreate"))
            {
            do_recreate(line);
            }
         else if (command.equals ("coordremoveglobalaction"))
            {
            do_coordremoveglobalaction (line);
            }
         else if (command.equals ("isequal"))
            {
            do_isequal (line);
            }
         else if (command.equals ("test"))
            {
            // need to add in variable substitution to allow
            // testing of runtime determined stuff such as globalid's
            while (line.size () > 0)
               _testResult.test ((String) line.remove (0));
            }
         else if (command.equals ("testnot"))
            {
            // need to add in variable substitution to allow
            // testing of runtime determined stuff such as globalid's
            while (line.size () > 0)
               _testResult.testNot ((String) line.remove (0));
            }
         else if (command.equals ("testnotall"))
            {
            // need to add in variable substitution to allow
            // testing of runtime determined stuff such as globalid's
            while (line.size () > 0)
               _testResult.testNotAll ((String) line.remove (0));
            }
         else if (command.equals ("testapplicable"))
            {
            // need to add in variable substitution to allow
            // testing of runtime determined stuff such as globalid's
            while (line.size () > 0)
               if (_testResult.testApplicable ((String) line.remove (0)))
                  {
                  terminate();
                  return true;
                  }
            }
         else if (command.equals ("trace"))
            {
            Trace.on ();
            }
         else
            {
            throw new SyntaxError (command + " unknown");
            }
         // _testResult.log ("Command '" + command + "' completed ok");
         // could check for unused arguments here ...
         }
      catch (SyntaxError e)
         {
         _testResult.error ("Line " + lineno () + " : " + e.getMessage ());
         // terminate ();
         // break;
         _ref.println (e.getMessage ());
         }
      catch (Exception e)
         {
         String logmsg = "Command '" + command + "' threw exception ";
         // Activity Service may have sub-classed exceptions, so check for javax.activity excp
         if (e instanceof ActionErrorException)
            _testResult.log (logmsg + ActionErrorException.class.getName ());
         else if (e instanceof ActionNotFoundException)
            _testResult.log (logmsg + ActionNotFoundException.class.getName ());
         else if (e instanceof ActivityCompletedException)
            _testResult.log (logmsg + ActivityCompletedException.class.getName ());
         else if (e instanceof ActivityNotProcessedException)
            _testResult.log (logmsg + ActivityNotProcessedException.class.getName ());
         else if (e instanceof ActivityPendingException)
            _testResult.log (logmsg + ActivityPendingException.class.getName ());
         else if (e instanceof ActivityRequiredException)
            _testResult.log (logmsg + ActivityRequiredException.class.getName ());
         else if (e instanceof ContextPendingException)
            _testResult.log (logmsg + ContextPendingException.class.getName ());
         else if (e instanceof InvalidActivityException)
            _testResult.log (logmsg + InvalidActivityException.class.getName ());
         else if (e instanceof InvalidParentContextException)
            _testResult.log (logmsg + InvalidParentContextException.class.getName ());
         else if (e instanceof InvalidStateException)
            _testResult.log (logmsg + InvalidStateException.class.getName ());
         else if (e instanceof NoActivityException)
            _testResult.log (logmsg + NoActivityException.class.getName ());
         else if (e instanceof NoImplementException)
            _testResult.log (logmsg + NoImplementException.class.getName ());
         else if (e instanceof NotOriginatorException)
            _testResult.log (logmsg + NotOriginatorException.class.getName ());
         else if (e instanceof PropertyGroupUnknownException)
            _testResult.log (logmsg + PropertyGroupUnknownException.class.getName ());
         else if (e instanceof ServiceAlreadyRegisteredException)
            _testResult.log (logmsg + ServiceAlreadyRegisteredException.class.getName ());
         else if (e instanceof ServiceNotRegisteredException)
            _testResult.log (logmsg + ServiceNotRegisteredException.class.getName ());
         else if (e instanceof SignalSetActiveException)
            _testResult.log (logmsg + SignalSetActiveException.class.getName ());
         else if (e instanceof SignalSetInactiveException)
            _testResult.log (logmsg + SignalSetInactiveException.class.getName ());
         else if (e instanceof SignalSetUnknownException)
            _testResult.log (logmsg + SignalSetUnknownException.class.getName ());
         else if (e instanceof SystemException)
            _testResult.log (logmsg + SystemException.class.getName ());
         else if (e instanceof TimeoutRangeException)
            _testResult.log (logmsg + TimeoutRangeException.class.getName ());
         else
            _testResult.log (logmsg + e.getClass ().getName ());

         _ref.println (e);
         _ref.println (e.getMessage ());
         Trace.exception (e);
         }

      return false;
      }  // end executeCmd


   /**
    * Sets the state of this thread to complete, so that the command
    * processing loop will complete and return.
    */
   public void finished ()
      {
      Trace.out ("TestActivity.finished");
      finished = true;
      }



   /**
    * Semaphore function to allow thread coordination.
    */
   public synchronized void acquire ()
      {
      Trace.out ("TestActivity.acquire");
      while (count == 0)
         {
         try {
            wait ();
            }
         catch (java.lang.InterruptedException e) { }
         }  // end while
      count --;
      }

   /**
    * Semaphore function to allow thread coordination.
    */
   public synchronized void release ()
      {
      Trace.out ("TestActivity.release");
      count++;
      notify ();
      }


   // MEMBER variables
   private int              count = 0;
   private boolean      finished  = false;

   // CLASS variables
   private static StreamTokenizer    rdr = null;
   private static Vector          tokens = new java.util.Vector ();
   private static Vector         threads = new java.util.Vector ();
   private static HashMap            tbl = new HashMap ();
   private static ServiceManagerImpl sm1 = null;
   private static TestResult _testResult = new TestResult ();
   private static PrintWriter       _log = null;
   private static PrintWriter       _ref = null;
   }  // end class TestActivity



/**
 * A utility class to encapsulate syntax/semantic errors in the
 * commands that are read and interpreted.
 */
final class SyntaxError extends Exception
   {
   public SyntaxError (String msg)
      {
      super (msg);
      }
   }  // end class SyntaxError
