/**********************************************************************/
/*                                                                    */
/* This program may be used, executed, copied, modified and           */
/* distributed without royalty for the purpose of developing, using,  */
/* marketing, or distributing.                                        */
/*                                                                    */
/**********************************************************************/
/*                                                                    */
/* DESCRIPTION:                                                       */
/* A utility class to be used to determine the output of a particular */
/* test.  The result of each execution step (and perhaps intermediate */
/* information) is logged and at the completion of the test, the log  */
/* can be compared to the expected results.  If it matches then the   */
/* result is logged ok else failure.  The test may also be invalidated*/
/* by calling the 'error' method.                                     */
/* A toString method is provided to enable the result to be printed.  */
/*                                                                    */
/**********************************************************************/

package com.ibm.activity.unittest;

import java.util.Vector;

/**
 * A Test logging and verification class.
 * During test execution, various information is logged.
 * This information can be verified as being expected using the
 * testXXX methods.
 * If any errors are logged, the the status is ERROR.
 * If any 'tests' fail, then the overall status is FAILED.
 * A test may be 'Not Applicable' ... a conditional test
 * Otherwise the result is PASS.
 */
final class TestResult implements com.ibm.activity.hls.TestLog
   {
   private  int      _result  = 0;   // 1 = error, 2 = fail, 3 = pass, 4 = n/a
   private  String   _msg     = null;
   private  Vector   _results = new Vector ();

   /**
    * Tests the last logged message.
    * The message is removed after testing.
    * If the Strings don't match, then the _result is set to FAIL.
    * If the Strings do match, then as long as the _result is not FAIL
    * or ERROR, then it is set to PASS
    *
    * @param testString   the String with which to perform testing.
    */
   public void test (String testString)
      {
      // if failed or error or n/a, then go no further
      if (_result == 1 || _result == 2 || _result == 4)
         return;

      // if testString null or _results mt then error 'invalid test'?
      if (testString == null)
         {
         error ("Null test string received");
         return;
         }

      // extract last result (remove from _results)
      String compareString = null;
      if (_results.size () > 0)
         {
         compareString = (String) _results.remove (_results.size () - 1);
         }

      // compare to testString
      // if equal and current status == 0 then passed "OK"
      if (testString.equals (compareString))
         {
         if (_result == 0)
            passed ("OK");
         }
      else //    ... expecting 'testString' and found 'string'
         failed ("Expecting '" + testString +
            "' and found '" + compareString + "'");

      }  // end test

   /**
    * Tests the last logged message.  The inverse of <code>test</code>.
    * The message is removed after testing.
    * If the Strings do match, then the _result is set to FAIL.
    * If the Strings don't match, then as long as the _result is not FAIL
    * or ERROR, then it is set to PASS
    *
    * @param testString   the String with which to perform testing.
    */
   public void testNot (String testString)
      {
      // if failed or error or n/a, then go no further
      if (_result == 1 || _result == 2 || _result == 4)
         return;

      // if testString null or _results mt then error 'invalid test'?
      if (testString == null)
         {
         error ("Null test string received");
         return;
         }

      // extract last result (remove from _results)
      String compareString = null;
      if (_results.size () > 0)
         {
         compareString = (String) _results.remove (_results.size () - 1);
         }

      // compare to testString
      // if NOT equal and current status == 0 then passed "OK"
      if (! testString.equals (compareString))
         {
         if (_result == 0)
            passed ("OK");
         }
      else //    ... not expecting 'testString'
         failed ("Not expecting '" + testString + "'");

      }  // end testNot

   /**
    * Tests the last logged message.  If matched then returns true;
    * The message is removed after testing.
    * If the Strings do match, then the _result is set to NOT APPLICABLE,
    * otherwise no change is made.
    *
    * @param testString   the String with which to perform testing.
    */
   public boolean testApplicable (String testString)
      {
      // if failed or error or n/a, then go no further
      if (_result == 1 || _result == 2 || _result == 4)
         return false;

      // if testString null or _results mt then error 'invalid test'?
      if (testString == null)
         {
         error ("Null test string received");
         return false;
         }

      // extract last result (remove from _results)
      String compareString = null;
      if (_results.size () > 0)
         {
         compareString = (String) _results.remove (_results.size () - 1);
         }

      // compare to testString
      // if equal then set N/A and return true
      if (testString.equals (compareString))
         {
         na("");
         return true;
         }

      return false;

      }  // end testApplicable

   /**
    * Tests against all logged message.
    * No messages are removed after testing.
    * If any Strings do match, then the _result is set to FAIL.
    * If no Strings match, then as long as the _result is not FAIL
    * or ERROR, then it is set to PASS
    *
    * @param testString   the String with which to perform testing.
    */
   public void testNotAll (String testString)
      {
      // if failed or error or n/a, then go no further
      if (_result == 1 || _result == 2 || _result == 4)
         return;

      // if testString null or _results mt then error 'invalid test'?
      if (testString == null)
         {
         error ("Null test string received");
         return;
         }

      boolean found = false;
      for (int i = 0; i < _results.size (); i++)
         {
         if (testString.equals ( _results.elementAt (i)))
            {
            found = true;
            break;
            }  // end if
         }  // end for

      if (! found)
         {
         if (_result == 0)
            passed ("OK");
         }
      else //    ... not expecting 'testString'
         failed ("Not expecting '" + testString + "'");

      }  // end testNotAll

   /**
    * TestLog interface method
    */
   public void log (String msg)
      {
      _results.add (msg);
      }  // end log

   /**
    * Debugging utility to print out all of the logged Strings.
    *
    * @param out   the stream to dump output to.
    */
   public void dump (java.io.PrintWriter out)
      {
      for (int i = 0; i < _results.size (); i++)
         out.println (_results.elementAt (i));
      }  // end dump

   /**
    * Sets the test status to FAILED with an appropriate message.
    *
    * @param msg   the message associated with the failure.
    */
   private void failed (String msg)
      {
      _msg    = msg;
      _result = 2;
      }  // end failed

   /**
    * Sets the test status to PASS with an appropriate message.
    *
    * @param msg   the message associated with the success.
    */
   private void passed (String msg)
      {
      _msg    = msg;
      _result = 3;
      }  // end passed

   /**
    * Sets the test status to ERROR with an appropriate message.
    *
    * @param msg   the message associated with the test error.
    */
   public void error (String msg)
      {
      _msg    = msg;
      _result = 1;
      }  // end error

   /**
    * Sets the test status to NOT APPLICABLE with an appropriate message.
    *
    * @param msg   the message associated with the test error.
    */
   public void na (String msg)
      {
      _msg    = msg;
      _result = 4;
      }  // end na

   /**
    * Returns a String representation of the test status.
    * The result can be parsed to separate the success/failure status
    * from the diagnostic message that is appended.
    */
   public String toString ()
      {
      switch (_result)
         {
      case 1:
         return "Test ERROR: " + _msg;
      case 2:
         return "Test FAILED: " + _msg;
      case 3:
         return "Test PASSED: " + _msg;
      case 4:
         return "Test NOT APPLICABLE: " + _msg;
      default:
         return "Test ERROR: No result set";
         }  // end switch

      }  // end toString

   }  // end class TestResult
